@@ -48,22 +48,51 @@
  */
 package com.openexchange.usm.contenttypes.mail.impl;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.BitSet;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.logging.Log;
-import org.json.*;
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
 
-import com.openexchange.usm.api.contenttypes.*;
-import com.openexchange.usm.api.exceptions.*;
-import com.openexchange.usm.api.session.*;
-import com.openexchange.usm.contenttypes.util.*;
-import com.openexchange.usm.ox_json.*;
+import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.MailContentType;
+import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
+import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
+import com.openexchange.usm.api.exceptions.InternalUSMException;
+import com.openexchange.usm.api.exceptions.OXCommunicationException;
+import com.openexchange.usm.api.exceptions.USMException;
+import com.openexchange.usm.api.exceptions.USMIllegalArgumentException;
+import com.openexchange.usm.api.exceptions.USMStartupException;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.Folder;
+import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.api.session.SyncResult;
+import com.openexchange.usm.contenttypes.util.AbstractTransferHandler;
+import com.openexchange.usm.contenttypes.util.USMContentTypesUtilErrorCodes;
+import com.openexchange.usm.contenttypes.util.UtilConstants;
+import com.openexchange.usm.ox_json.JSONResult;
+import com.openexchange.usm.ox_json.JSONResultType;
+import com.openexchange.usm.ox_json.OXJSONAccess;
+import com.openexchange.usm.ox_json.OXResource;
 
 public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler {
 
+	private static final String DROP_PREFIX = "dropPrefix";
+	private static final String COM_OPENEXCHANGE_MAIL_CONVERSION_SEQUENCEID = "com.openexchange.mail.conversion.sequenceid";
+	private static final String COM_OPENEXCHANGE_MAIL_CONVERSION_MAILID = "com.openexchange.mail.conversion.mailid";
+	private static final String COM_OPENEXCHANGE_MAIL_CONVERSION_FULLNAME = "com.openexchange.mail.conversion.fullname";
+	private static final String COM_OPENEXCHANGE_MAIL_ICAL = "com.openexchange.mail.ical";
 	private static final String FORCE = "force";
 	private static final String FOLDER = "folder";
-	private static final String TEXT = "text";
 	private static final String ONE = "1";
 	private static final String HARDDELETE = "harddelete";
 	private static final String RAW = "raw";
@@ -72,7 +101,6 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 	private static final String UNSEEN = "unseen";
 	private static final String SRC = "src";
 	private static final String SAVE = "save";
-	private static final String ZERO = "0";
 
 	private final BitSet _getAllBitSet = new BitSet();
 	private final int _idIndex;
@@ -106,7 +134,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 
 	public Object getMailAttachment(Session session, String folderId, String mailId, String attachmentId)
 			throws USMException {
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UtilConstants.FOLDER, folderId);
 		parameters.put(UtilConstants.ID, mailId);
 		parameters.put(MailConstants.ATTACHMENT, attachmentId);
@@ -120,7 +148,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 	public String createNewMail(String folderId, int flag, String mailMIMEdataBlock, Session session)
 			throws USMException {
 		//TODO: use import instead of new (see svn history!)
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(SRC, ONE);
 		parameters.put(UtilConstants.FOLDER, folderId);
 		parameters.put(FLAGS, String.valueOf(flag));
@@ -137,17 +165,18 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		}
 	}
 
-	public DataObject getReplyForward(Session session, String messageID, String folderID, String action)
-			throws USMException {
+	public DataObject getReplyForward(Session session, String messageID, String folderID, String action, String view,
+			boolean dropPrefix) throws USMException {
 		DataObject mail = _contentType.newDataObject(session);
 		mail.setID(messageID);
 		mail.setParentFolderID(folderID);
 		mail.commitChanges();
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UtilConstants.ID, messageID);
 		parameters.put(UtilConstants.FOLDER, folderID);
 		parameters.put(UNSEEN, ONE);
-		parameters.put(VIEW, TEXT);
+		parameters.put(VIEW, view);
+		parameters.put(DROP_PREFIX, String.valueOf(dropPrefix));
 
 		JSONResult result = _ajaxAccess.doGet(UtilConstants.MAIL_PATH, action, session, parameters);
 		if (result.getResultType() == JSONResultType.Error)
@@ -155,6 +184,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 					.getJSONObject());
 		BitSet reqFields = createRequestedFields(_contentType);
 		reqFields.set(18); //attachments
+		reqFields.set(17); //headers
 		updateDataObjectFromJSONResult(mail, reqFields, result);
 		mail.setID(messageID);
 		mail.setParentFolderID(folderID);
@@ -182,7 +212,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		if (_journal.isDebugEnabled())
 			_journal.debug(object.getSession() + " Delete mail " + object.getID());
 		checkTypeToWrite(object, DefaultContentTypes.MAIL_ID);
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UtilConstants.TIMESTAMP, String.valueOf(object.getTimestamp()));
 		if (hardDelete)
 			parameters.put(HARDDELETE, ONE);
@@ -205,7 +235,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 	@Override
 	public DataObject[] readUpdatedFolderContent(Folder folder, BitSet requestedFields, long timestamp)
 			throws USMException {
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		//XXX Not IMAP - Response: Just an empty JSON array is going to be returned since this action cannot be applied to IMAP. 
 
 		//parameters.put(UtilConstants.FOLDER, folder.getID());
@@ -231,7 +261,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 	@Override
 	public void writeUpdatedDataObject(DataObject object, long timestamp) throws USMException {
 		checkTypeToWrite(object, DefaultContentTypes.MAIL_ID);
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 
 		parameters.put(UtilConstants.ID, object.getID());
 		parameters.put(UtilConstants.FOLDER, object.getOriginalParentFolderID());
@@ -241,7 +271,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		Number color = (Number) object.getFieldContent(MailConstants.MAIL_UPDATE_COLOR_LABEL);
 		if (object.isFieldModified(MailConstants.MAIL_UPDATE_FLAGS)) {
 			Number flagValue = (Number) object.getFieldContent(MailConstants.MAIL_UPDATE_FLAGS);
-			int flagsToSet = flagValue == null ? 0 : flagValue.intValue();
+			int flagsToSet = (flagValue == null) ? 0 : flagValue.intValue();
 			int flagsToRemove = MailContentType.ALL_FLAGS - flagsToSet;
 			if (flagsToSet > 0)
 				updateFlags(object, parameters, color, flagsToSet, true);
@@ -302,7 +332,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 
 	@Override
 	public void readDataObject(DataObject object, BitSet requestedFields) throws USMException {
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UNSEEN, ONE);
 		parameters.put(VIEW, RAW);
 		readDataObject(object, requestedFields, parameters);
@@ -312,7 +342,7 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		if (_journal.isDebugEnabled())
 			_journal.debug(object.getSession() + " Read structured mail with id " + object.getID());
 		checkTypeToWrite(object, _contentType.getID());
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UNSEEN, ONE);
 		parameters.put(UtilConstants.ID, object.getID());
 		parameters.put(UtilConstants.FOLDER, object.getParentFolderID());
@@ -329,14 +359,18 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		return resultObject;
 	}
 
-	public byte[] readMailSource(DataObject object, BitSet requestedFields) throws USMException {
+	public byte[] readMailSource(DataObject object, BitSet requestedFields, boolean ignoreICalAttachments)
+			throws USMException {
 		if (_journal.isDebugEnabled())
-			_journal.debug(object.getSession() + " Read mail with id " + object.getID());
+			_journal.debug(object.getSession() + " Read mail with id " + object.getID()
+					+ " with ignore ical attachments param: " + ignoreICalAttachments);
 		checkTypeToWrite(object, _contentType.getID());
-		HashMap<String, String> parameters = new HashMap<String, String>();
+		Map<String, String> parameters = new HashMap<String, String>();
 		parameters.put(UNSEEN, ONE);
 		parameters.put(SRC, ONE);
 		parameters.put(SAVE, ONE);
+		if (ignoreICalAttachments)
+			parameters.put("ignorable", "ics");
 		parameters.put(UtilConstants.ID, object.getID());
 		parameters.put(UtilConstants.FOLDER, object.getParentFolderID());
 		parameters.put(UtilConstants.ACTION, UtilConstants.ACTION_GET);
@@ -351,65 +385,62 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 	@Override
 	public DataObject[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
 		if (AbstractTransferHandler.CHECK_ACCESS_RIGHTS_BEFORE_FOLDER_READ && !hasFolderReadRights(folder))
-			return EMPTY_LIST;
+			return SyncResult.EMPTY_DATA_OBJECT_ARRAY;
 		Session session = folder.getSession();
 		long startDate = session.getStartDate();
 		long endDate = session.getEndDate();
 		int mailLimit = session.getMailLimit();
-		if (startDate == 0L && endDate == Long.MAX_VALUE && mailLimit <= 0)
-			return super.readFolderContent(folder, requestedFields);
-		DataObject[] idList = super.readFolderContent(folder, _getAllBitSet);
-		List<DataObject> filteredList = new ArrayList<DataObject>();
-		for (DataObject o : idList) {
-			Long t = (Long) o.getFieldContent(_receivedDateIndex);
-			if (t == null || (t >= startDate && t <= endDate))
-				filteredList.add(o);
-		}
-		if (filteredList.isEmpty())
-			return EMPTY_LIST;
-		if (filteredList.size() > mailLimit) {
-			Collections.sort(filteredList, new Comparator<DataObject>() {
-				public int compare(DataObject a, DataObject b) {
-					// Sort descending, if no time is present, treat as newest
-					Long ta = (Long) a.getFieldContent(_receivedDateIndex);
-					Long tb = (Long) b.getFieldContent(_receivedDateIndex);
-					if (ta == null) {
-						return (tb == null) ? 0 : -1;
-					} else if (tb == null) {
-						return 1;
-					} else {
-						return tb.compareTo(ta);
-					}
-				}
-			});
-			filteredList.subList(mailLimit, filteredList.size()).clear();
-		}
-		JSONArray ids = new JSONArray();
-		for (DataObject o : filteredList) {
-			JSONObject element = new JSONObject();
-			try {
-				element.put(FOLDER, folder.getID());
-				element.put(UtilConstants.ID, o.getID());
-			} catch (JSONException e) {
-				// Ignored because it can not happen
-			}
-			ids.put(element);
-		}
-		HashMap<String, String> parameters = new HashMap<String, String>();
-		parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
 		try {
+			if (startDate == 0L && endDate == Long.MAX_VALUE && mailLimit <= 0)
+				return super.readFolderContent(folder, requestedFields);
+			DataObject[] idList = super.readFolderContent(folder, _getAllBitSet);
+			List<DataObject> filteredList = new ArrayList<DataObject>();
+			for (DataObject o : idList) {
+				Long t = (Long) o.getFieldContent(_receivedDateIndex);
+				if (t == null || (t >= startDate && t <= endDate))
+					filteredList.add(o);
+			}
+			if (filteredList.isEmpty())
+				return SyncResult.EMPTY_DATA_OBJECT_ARRAY;
+			if (mailLimit > 0 && filteredList.size() > mailLimit) {
+				Collections.sort(filteredList, new Comparator<DataObject>() {
+					public int compare(DataObject a, DataObject b) {
+						// Sort descending, if no time is present, treat as newest
+						Long ta = (Long) a.getFieldContent(_receivedDateIndex);
+						Long tb = (Long) b.getFieldContent(_receivedDateIndex);
+						if (ta == null) {
+							return (tb == null) ? 0 : -1;
+						} else if (tb == null) {
+							return 1;
+						} else {
+							return tb.compareTo(ta);
+						}
+					}
+				});
+				filteredList.subList(mailLimit, filteredList.size()).clear();
+			}
+			JSONArray ids = new JSONArray();
+			for (DataObject o : filteredList) {
+				JSONObject element = new JSONObject();
+				try {
+					element.put(FOLDER, folder.getID());
+					element.put(UtilConstants.ID, o.getID());
+				} catch (JSONException ignored) {
+					// Ignored because it can not happen
+				}
+				ids.put(element);
+			}
+			Map<String, String> parameters = new HashMap<String, String>();
+			parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
 			JSONResult result = _ajaxAccess.doPut(getOXAjaxAccessPath(), UtilConstants.ACTION_LIST, session,
 					parameters, ids);
 			DataObject[] listOfResults = getListOfResults(folder, requestedFields, result);
 			return extractDeletedMails(session, listOfResults);
 		} catch (OXCommunicationException e) {
 			//catch the OXCommunication exception and than get the access rights
-			if (hasFolderReadRights(folder)) {
+			if (hasFolderReadRights(folder))
 				throw e;
-			} else {
-				DataObject[] emptyList = new DataObject[0];
-				return emptyList;
-			}
+			return SyncResult.EMPTY_DATA_OBJECT_ARRAY;
 		}
 	}
 
@@ -427,4 +458,157 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
 		}
 		return listOfResults;
 	}
+
+	public JSONObject convertICalToJSON(DataObject mailObject, String attSeqId, String timezone) throws USMException {
+		if (_journal.isDebugEnabled())
+			_journal.debug("Converting iCal attachment to JSONObject for mail: " + mailObject.getID());
+
+		JSONObject conversionObject = createConversionDataSource(mailObject, attSeqId);
+
+		JSONObject dataHandler = new JSONObject();
+		try {
+			dataHandler.put("identifier", "com.openexchange.ical.json");
+			if (timezone != null) {
+				JSONArray args = new JSONArray();
+				JSONObject timezoneObj = new JSONObject();
+				timezoneObj.put("com.openexchange.groupware.calendar.timezone", timezone);
+				args.put(timezoneObj);
+				dataHandler.put("args", args);
+			}
+			conversionObject.put("datahandler", dataHandler);
+		} catch (JSONException e1) {
+			throw generateException(USMContentTypesMailErrorCodes.ERROR_ON_DATA_HANDLER_CREATE, mailObject,
+					"convert iCal to JSON: can not create data handler", e1);
+		}
+
+		JSONResult result = _ajaxAccess.doPut(UtilConstants.CONVERSION_PATH, UtilConstants.ACTION_CONVERT, mailObject
+				.getSession(), new HashMap<String, String>(), conversionObject);
+		checkResult(result, JSONResultType.JSONObject);
+		JSONObject resultObject = null;
+		try {
+			resultObject = result.getJSONObject().getJSONArray("data").getJSONObject(0);
+		} catch (JSONException e) {
+			throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER8, mailObject,
+					"convert iCal to JSON: data  not present", e);
+		}
+		return resultObject;
+	}
+
+	public JSONObject analyzeICalAttachment(DataObject mailObject, String attSeqId) throws USMException {
+		if (_journal.isDebugEnabled())
+			_journal.debug("Analyzing iCal attachment for mail: " + mailObject.getID());
+
+		JSONObject conversionObject = createITIPDataSource(mailObject, attSeqId);
+
+		Map<String, String> parameters = new HashMap<String, String>();
+		parameters.put("dataSource", COM_OPENEXCHANGE_MAIL_ICAL);
+		JSONResult result = _ajaxAccess.doPut(UtilConstants.ITIP_PATH, UtilConstants.ACTION_ANALYZE, mailObject
+				.getSession(), parameters, conversionObject);
+		checkResult(result, JSONResultType.JSONObject);
+		JSONObject resultObject = null;
+		try {
+			resultObject = result.getJSONObject().getJSONArray("data").getJSONObject(0);
+		} catch (JSONException e) {
+			throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER9, mailObject,
+					"analyze iCal: data  not present", e);
+		}
+		return resultObject;
+	}
+
+	public JSONObject updateObjectBasedOnMeetingRequest(DataObject mailObject, String attSeqId, String action)
+			throws USMException {
+		if (_journal.isDebugEnabled())
+			_journal.debug("Updating object based on iCal attachment for mail: " + mailObject.getID() + ", action: "
+					+ action);
+
+		JSONObject conversionObject = createITIPDataSource(mailObject, attSeqId);
+
+		Map<String, String> parameters = new HashMap<String, String>();
+		parameters.put("dataSource", COM_OPENEXCHANGE_MAIL_ICAL);
+		JSONResult result = _ajaxAccess.doPut(UtilConstants.ITIP_PATH, action, mailObject.getSession(), parameters,
+				conversionObject);
+		checkResult(result, JSONResultType.JSONObject);
+		try {
+			JSONArray dataArray = result.getJSONObject().getJSONArray("data");
+			return dataArray.length() > 0 ? dataArray.getJSONObject(0) : null;
+		} catch (JSONException e) {
+			throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER9, mailObject,
+					"analyze iCal: data  not present", e);
+		}
+	}
+
+	public JSONObject respondToMeetingRequest(DataObject mailObject, String attSeqId, int confirmation)
+			throws USMException {
+		if (_journal.isDebugEnabled())
+			_journal.debug("Accepting meeting request for mail: " + mailObject.getID());
+
+		JSONObject conversionObject = createITIPDataSource(mailObject, attSeqId);
+
+		Map<String, String> parameters = new HashMap<String, String>();
+		parameters.put("dataSource", COM_OPENEXCHANGE_MAIL_ICAL);
+
+		String action = null;
+		switch (confirmation) {
+			case 1:
+				action = UtilConstants.ACTION_ACCEPT;
+				break;
+			case 2:
+				action = UtilConstants.ACTION_DECLINE;
+				break;
+			case 3:
+				action = UtilConstants.ACTION_TENTATIVE;
+				break;
+		}
+		JSONResult result = _ajaxAccess.doPut(UtilConstants.ITIP_PATH, action, mailObject.getSession(), parameters,
+				conversionObject);
+
+		checkResult(result, JSONResultType.JSONObject);
+		JSONObject resultObject = null;
+		try {
+			resultObject = result.getJSONObject().getJSONArray("data").getJSONObject(0);
+		} catch (JSONException e) {
+			throw generateException(USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER9, mailObject,
+					"analyze iCal: data  not present", e);
+		}
+
+		return resultObject;
+	}
+
+	private JSONObject createConversionDataSource(DataObject mailObject, String attSeqId)
+			throws OXCommunicationException {
+		JSONObject conversionObject = new JSONObject();
+		JSONObject dataSource = new JSONObject();
+		try {
+			dataSource.put("identifier", COM_OPENEXCHANGE_MAIL_ICAL);
+			JSONArray args = new JSONArray();
+			JSONObject folder = new JSONObject();
+			folder.put(COM_OPENEXCHANGE_MAIL_CONVERSION_FULLNAME, mailObject.getParentFolderID());
+			args.put(0, folder);
+			JSONObject mailid = new JSONObject();
+			mailid.put(COM_OPENEXCHANGE_MAIL_CONVERSION_MAILID, mailObject.getID());
+			args.put(1, mailid);
+			JSONObject seqid = new JSONObject();
+			seqid.put(COM_OPENEXCHANGE_MAIL_CONVERSION_SEQUENCEID, attSeqId);
+			args.put(2, seqid);
+			dataSource.put("args", args);
+			conversionObject.put("datasource", dataSource);
+		} catch (JSONException e1) {
+			throw generateException(USMContentTypesMailErrorCodes.ERROR_ON_DATA_SOURCE_CREATE, mailObject,
+					"convert iCal to JSON: can not create data source", e1);
+		}
+		return conversionObject;
+	}
+
+	private JSONObject createITIPDataSource(DataObject mailObject, String attSeqId) throws OXCommunicationException {
+		JSONObject conversionObject = new JSONObject();
+		try {
+			conversionObject.put(COM_OPENEXCHANGE_MAIL_CONVERSION_FULLNAME, mailObject.getParentFolderID());
+			conversionObject.put(COM_OPENEXCHANGE_MAIL_CONVERSION_MAILID, mailObject.getID());
+			conversionObject.put(COM_OPENEXCHANGE_MAIL_CONVERSION_SEQUENCEID, attSeqId);
+		} catch (JSONException e1) {
+			throw generateException(USMContentTypesMailErrorCodes.ERROR_ON_DATA_SOURCE_CREATE_2, mailObject,
+					"analyze iCal: can not create data source", e1);
+		}
+		return conversionObject;
+	}
 }
