@@ -52,8 +52,10 @@ import java.io.*;
 import java.text.SimpleDateFormat;
 import java.util.*;
 
+import javax.activation.DataHandler;
 import javax.mail.*;
 import javax.mail.internet.*;
+import javax.mail.util.ByteArrayDataSource;
 
 import org.json.*;
 
@@ -75,6 +77,8 @@ import com.openexchange.usm.util.Toolkit;
  */
 public class MimeMailBuilder {
 
+	private static final String X_ORIGINAL_HEADERS = "x-original-headers";
+
 	private static final String READ_DATE = "read-date";
 
 	private static final String UTF_8 = "UTF-8";
@@ -102,12 +106,12 @@ public class MimeMailBuilder {
 	private static final String BODY = "body";
 	private static final String DATA = "data";
 
+	private static final String SMIME_BODY_DATA = "smime_body_data";
+
 	private static final String EMPTY_STR = "";
 
 	private static final char SEMICOLON = ';';
 
-	private static final String NAME = "name";
-
 	private static final String CONTENT_TYPE_UPPER = "Content-Type";
 	private static final String CONTENT_TRANSFER_ENCODING_UPPER = "Content-Transfer-Encoding";
 
@@ -140,14 +144,17 @@ public class MimeMailBuilder {
 	private static final String CONTENTTYPE_TEXT_PREFIX = "text/";
 	private static final String CONTENTTYPE_TEXT_PLAIN = "plain";
 	private static final String CONTENTTYPE_RFC822 = "message/rfc822";
-	private static final String CONTENTTYPE_APPLICATION_OCTETSTREAM = "application/octet-stream";
-
-	private static final String CONTENT_DISPOSITION_ATTACHMENT = "ATTACHMENT";
 
 	private static final String CHARSET = "charset";
 
+	private static final String CHARSET_PARAMETER = CHARSET + "=";
+
+	private final SimpleDateFormat _dateFormat = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.ENGLISH);
+
 	private USMMimeMessage _message;
 
+	private boolean _isReplyOrForward;
+
 	//----------------------------------------------------------------------------------------------------
 	// Begin: with JAVA Mail API
 
@@ -187,13 +194,46 @@ public class MimeMailBuilder {
 
 	}
 
-	public String convertJSONObjectToMimeMail(JSONObject data) throws USMJSONAPIException {
-		buildMimeMailFromJSONObject(data);
-		return writeTo();
+	public String convertJSONObjectToMimeMail(JSONObject data, boolean isReplyOrForward) throws USMJSONAPIException {
+		_isReplyOrForward = isReplyOrForward;
+		if (data.has(SMIME_BODY_DATA)) {
+			if (data.has(BODY))
+				throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_BODY_FIELDS,
+						ResponseStatusCode.BAD_REQUEST, "Message contains smime body field and other body fields!");
+			return buildSmimeMailFromJSONObject(data);
+		} else {
+			buildMimeMailFromJSONObject(data);
+			return writeTo();
+		}
 	}
 
-	private MimeMultipart buildMimePartForMultipart(JSONObject data, String contentTypeInfo, String subtype)
-			throws MessagingException, JSONException, USMJSONAPIException {
+	private String buildSmimeMailFromJSONObject(JSONObject data) throws USMJSONAPIException {
+		JSONObject headers = data.optJSONObject(HEADERS_FIELDNAME);
+		StringBuilder sb = new StringBuilder(1000);
+		try {
+			Object contentType;
+			contentType = (headers != null && headers.has(CONTENT_TYPE)) ? headers.get(CONTENT_TYPE) : null;
+			String contentTypeOfPart = buildContentTypeAsString(contentType);
+			//add headers to message
+			MimePart part = new MimeBodyPart();
+			appendHeadersToPart(part, headers, contentTypeOfPart);
+			for (Enumeration<?> e = part.getAllHeaderLines(); e.hasMoreElements();) {
+				sb.append((String) e.nextElement());
+				sb.append("\r\n");
+			}
+			sb.append(new String(Toolkit.decodeBase64(data.getString(SMIME_BODY_DATA))));
+		} catch (JSONException e) {
+			throw new USMJSONAPIException(ConnectorBundleErrorCodes.SMIMEMAIL_JSONEXCEPTION_ERROR,
+					ResponseStatusCode.WRONG_MISSING_PARAMETERS, "JSONException: " + e.getMessage());
+		} catch (MessagingException e) {
+			throw new USMJSONAPIException(ConnectorBundleErrorCodes.SMIMEMAIL_JAVAPI_MESSAGING_ERROR,
+					ResponseStatusCode.INTERNAL_ERROR, "error by building of body part: " + e.getMessage());
+		}
+		return sb.toString();
+	}
+
+	private MimeMultipart buildMimePartForMultipart(JSONObject data, String subtype) throws MessagingException,
+			JSONException, USMJSONAPIException {
 
 		MimeMultipart multiPart = new MimeMultipart(subtype);
 		if (MULTIPART_MIXED.equals(subtype))
@@ -204,7 +244,8 @@ public class MimeMailBuilder {
 		if (parts == null)
 			return multiPart;
 
-		for (int i = 0; i < parts.length(); i++) {
+		int length = parts.length();
+		for (int i = 0; i < length; i++) {
 			JSONObject jsonpart = parts.getJSONObject(i);
 			MimeBodyPart part = new MimeBodyPart();
 			buildPart(part, jsonpart);
@@ -216,64 +257,59 @@ public class MimeMailBuilder {
 
 	private void buildPart(MimePart part, JSONObject json) throws USMJSONAPIException {
 		try {
-			JSONObject headers = json.getJSONObject(HEADERS_FIELDNAME);
-			Object contentType = headers.has(CONTENT_TYPE) ? headers.get(CONTENT_TYPE) : CONTENTTYPE_TEXT_PREFIX + CONTENTTYPE_TEXT_PLAIN + "; "  + CHARSET + "=" + UTF_8;
-			String contentTypeOfPart = buildContentTypeAsString(headers, contentType);
+			JSONObject headers = json.optJSONObject(HEADERS_FIELDNAME);
+			Object contentType = (headers != null && headers.has(CONTENT_TYPE)) ? headers.get(CONTENT_TYPE)
+					: CONTENTTYPE_TEXT_PREFIX + CONTENTTYPE_TEXT_PLAIN + "; " + CHARSET_PARAMETER + UTF_8;
+			String contentTypeOfPart = buildContentTypeAsString(contentType);
 			if (contentTypeOfPart.startsWith(CONTENTTYPE_MULTIPART_MIXED)) {
 				//add headers to message
 				appendHeadersToPart(part, headers, CONTENTTYPE_MULTIPART_MIXED);
 				// set multi part content
-				part.setContent(buildMimePartForMultipart(json, contentTypeOfPart, MULTIPART_MIXED),
-						CONTENTTYPE_MULTIPART_MIXED);
+				part.setContent(buildMimePartForMultipart(json, MULTIPART_MIXED), CONTENTTYPE_MULTIPART_MIXED);
 			} else if (contentTypeOfPart.startsWith(CONTENTTYPE_MULTIPART_ALTER)) {
 				//add headers
 				appendHeadersToPart(part, json.getJSONObject(HEADERS_FIELDNAME), CONTENTTYPE_MULTIPART_ALTER);
 				// set multi part content
-				part.setContent(buildMimePartForAlternative(json, contentTypeOfPart), CONTENTTYPE_MULTIPART_ALTER);
+				part.setContent(buildMimePartForAlternative(json), CONTENTTYPE_MULTIPART_ALTER);
 			} else if (contentTypeOfPart.startsWith(MULTIPART)) {
 				//add headers to message
 				appendHeadersToPart(part, headers, contentTypeOfPart);
 				// set multi part content
 				String subtype = contentTypeOfPart.replaceFirst(MULTIPART, "");
-				part.setContent(buildMimePartForMultipart(json, contentTypeOfPart, subtype), contentTypeOfPart);
+				part.setContent(buildMimePartForMultipart(json, subtype), contentTypeOfPart);
+
 			} else {
-				JSONObject body = json.has(BODY)? json.getJSONObject(BODY) : null;
-				if (headers.has(CONTENT_DISPOSITION)
-						&& headers.getString(CONTENT_DISPOSITION).toUpperCase().contains(CONTENT_DISPOSITION_ATTACHMENT)) {
-					int posName = contentTypeOfPart.indexOf(NAME);
-					String mailcontentTypeAttachment = contentTypeOfPart.substring(0, posName).trim();
-					int posSep2 = mailcontentTypeAttachment.indexOf(SEMICOLON);
-					if (posSep2 > 0)
-						mailcontentTypeAttachment = mailcontentTypeAttachment.substring(0, posSep2);
-					//add headers
-					appendHeadersToPart(part, headers, contentTypeOfPart);
-					// set content
-					String content = body != null && body.has(DATA) ? body.getString(DATA) : "";
-					String encoding = getValue(headers, CONTENT_TRANSFER_ENCODING, 0, null);
-					if (BASE64_ENCODING.equals(encoding)) {
-						if (mailcontentTypeAttachment.startsWith(CONTENTTYPE_TEXT_PREFIX))
-							mailcontentTypeAttachment = CONTENTTYPE_APPLICATION_OCTETSTREAM;
-						part.setContent(Toolkit.decodeBase64(content), mailcontentTypeAttachment);
-						appendHeaderConditional(part, CONTENT_TRANSFER_ENCODING_UPPER, encoding);
-					} else {
-						part.setContent(content, mailcontentTypeAttachment);
-					}
-					part.setHeader(CONTENT_TYPE_UPPER, contentTypeOfPart);
-				} else if (contentTypeOfPart.startsWith(CONTENTTYPE_TEXT_PREFIX)) {
+				JSONObject body = json.has(BODY) ? json.getJSONObject(BODY) : null;
+				if (contentTypeOfPart.startsWith(CONTENTTYPE_TEXT_PREFIX)) {
 					try {
 						//add headers
 						String encoding = getValue(headers, CONTENT_TRANSFER_ENCODING, 0, null);
 						appendHeaderConditional(part, CONTENT_TRANSFER_ENCODING_UPPER, encoding);
 						appendHeadersToPart(part, headers, contentTypeOfPart);
 						//build text part
-						String content = body != null && body.has(DATA) ? body.getString(DATA) : "";
+						String content = (body != null && body.has(DATA)) ? body.getString(DATA) : "";
 						buildTextPart(part, contentTypeOfPart, content);
 					} catch (UnsupportedEncodingException e) {
-						throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR,
+						throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR_3,
 								ResponseStatusCode.INTERNAL_ERROR, "unsupported encoding of text content: "
 										+ e.getMessage());
 
 					}
+				} else if (body != null && body.has(DATA)) {
+					String mailcontentTypeAttachment = extractPlainContentType(contentTypeOfPart).trim();
+					//add headers
+					appendHeadersToPart(part, headers, contentTypeOfPart);
+					// set content
+					String content = body.getString(DATA);
+					String encoding = getValue(headers, CONTENT_TRANSFER_ENCODING, 0, null);
+					if (BASE64_ENCODING.equals(encoding)) {
+						appendHeaderConditional(part, CONTENT_TRANSFER_ENCODING_UPPER, encoding);
+						part.setDataHandler(new DataHandler(new ByteArrayDataSource(Toolkit.decodeBase64(content),
+								mailcontentTypeAttachment)));
+					} else {
+						part.setContent(content, mailcontentTypeAttachment);
+					}
+					part.setHeader(CONTENT_TYPE_UPPER, contentTypeOfPart);
 				} else if (contentTypeOfPart.startsWith(CONTENTTYPE_RFC822)) {
 					appendHeadersToPart(part, headers, contentTypeOfPart);
 					USMMimeMessage nestedPart = new USMMimeMessage(Session.getInstance(new Properties()));
@@ -292,16 +328,27 @@ public class MimeMailBuilder {
 
 	}
 
-	private String buildContentTypeAsString(JSONObject headers, Object paramHeader) throws JSONException,
-			USMJSONAPIException {
+	private String extractPlainContentType(String contentTypeOfPart) {
+		int len = contentTypeOfPart.length();
+		for (int i = 0; i < len; i++) {
+			char c = contentTypeOfPart.charAt(i);
+			if (c == SEMICOLON || Character.isWhitespace(c))
+				return contentTypeOfPart.substring(0, i);
+		}
+		return contentTypeOfPart;
+	}
+
+	private String buildContentTypeAsString(Object header) throws JSONException, USMJSONAPIException {
+		if (header == null)
+			return null;
 		ContentType contentType = new ContentType();
 		try {
-			if (paramHeader instanceof String) {
-				contentType = new ContentType((String) paramHeader);
-			} else if (paramHeader instanceof JSONObject) {
-				String type = ((JSONObject) paramHeader).getString(CONTENTTYPE_TYPE);
+			if (header instanceof String) {
+				contentType = new ContentType((String) header);
+			} else if (header instanceof JSONObject) {
+				String type = ((JSONObject) header).getString(CONTENTTYPE_TYPE);
 				contentType.setBaseType(type);
-				JSONObject params = (((JSONObject) paramHeader).has(CONTENTTYPE_PARAMS)) ? ((JSONObject) paramHeader)
+				JSONObject params = (((JSONObject) header).has(CONTENTTYPE_PARAMS)) ? ((JSONObject) header)
 						.getJSONObject(CONTENTTYPE_PARAMS) : new JSONObject();
 				for (Iterator<?> iterator = params.keys(); iterator.hasNext();) {
 					String key = (String) iterator.next();
@@ -322,16 +369,15 @@ public class MimeMailBuilder {
 		return contentType.toString();
 	}
 
-	private String buildContentDispositionAsString(JSONObject headers, Object paramHeader) throws JSONException,
-			USMJSONAPIException {
+	private String buildContentDispositionAsString(Object header) throws JSONException, USMJSONAPIException {
 		ContentDisposition contentDisp = new ContentDisposition();
 		try {
-			if (paramHeader instanceof String)
-				contentDisp = new ContentDisposition((String) headers.getString(CONTENT_DISPOSITION));
-			else if (paramHeader instanceof JSONObject) {
-				String type = ((JSONObject) paramHeader).getString(CONTENTTYPE_TYPE);
+			if (header instanceof String)
+				contentDisp = new ContentDisposition((String) header);
+			else if (header instanceof JSONObject) {
+				String type = ((JSONObject) header).getString(CONTENTTYPE_TYPE);
 				contentDisp.setDisposition(type);
-				JSONObject params = (((JSONObject) paramHeader).has(CONTENTTYPE_PARAMS)) ? ((JSONObject) paramHeader)
+				JSONObject params = (((JSONObject) header).has(CONTENTTYPE_PARAMS)) ? ((JSONObject) header)
 						.getJSONObject(CONTENTTYPE_PARAMS) : new JSONObject();
 				for (Iterator<?> iterator = params.keys(); iterator.hasNext();) {
 					String key = (String) iterator.next();
@@ -354,19 +400,17 @@ public class MimeMailBuilder {
 
 	private void buildTextPart(MimePart part, String contentTypeOfPart, String decodedContent) throws JSONException,
 			MessagingException, UnsupportedEncodingException {
+		String mailcontentTypeText = extractPlainContentType(contentTypeOfPart).trim();
 		int posCharset = contentTypeOfPart.indexOf(CHARSET);
-		String mailcontentTypeText = contentTypeOfPart.substring(0, posCharset).trim();
-		int posSep2 = mailcontentTypeText.indexOf(SEMICOLON);
-		if (posSep2 > 0)
-			mailcontentTypeText = mailcontentTypeText.substring(0, posSep2);
-		String charset = contentTypeOfPart.substring(posCharset, contentTypeOfPart.length());
-		int eqvPos = charset.indexOf('=');
-		charset = charset.substring(eqvPos + 1, charset.length()).trim();
+		int charSetBeginPos = posCharset + CHARSET.length() + 1;
+		int charSetEndPos = contentTypeOfPart.indexOf(';', charSetBeginPos);
+		String charset = ((charSetEndPos >= 0) ? contentTypeOfPart.substring(charSetBeginPos, charSetEndPos)
+				: contentTypeOfPart.substring(charSetBeginPos)).trim();
 		// set content
 		int posSubtype = mailcontentTypeText.indexOf('/');
 		String subtype = CONTENTTYPE_TEXT_PLAIN;
 		if (posSubtype > 0)
-			subtype = mailcontentTypeText.substring(posSubtype + 1, mailcontentTypeText.length());
+			subtype = mailcontentTypeText.substring(posSubtype + 1);
 		// set content
 		byte[] contentBytes;
 		//decode if Content-Transfer-Encoding = Base64
@@ -377,8 +421,8 @@ public class MimeMailBuilder {
 		part.setText(new String(contentBytes, charset), charset, subtype);
 	}
 
-	private MimeMultipart buildMimePartForAlternative(JSONObject data, String contentTypeInfo)
-			throws MessagingException, JSONException, USMJSONAPIException {
+	private MimeMultipart buildMimePartForAlternative(JSONObject data) throws MessagingException, JSONException,
+			USMJSONAPIException {
 
 		MimeMultipart multiPart = new MimeMultipart(MULTIPART_ALTERNATIVE);
 		// create and add parts
@@ -386,13 +430,14 @@ public class MimeMailBuilder {
 		if (parts == null)
 			return multiPart;
 		// build simple text parts: plain, html
-		for (int i = 0; i < parts.length(); i++) {
+		int length = parts.length();
+		for (int i = 0; i < length; i++) {
 			JSONObject jsonpart = parts.getJSONObject(i);
 			MimeBodyPart part = new MimeBodyPart();
 			JSONObject headers = jsonpart.getJSONObject(HEADERS_FIELDNAME);
 			//build text part
 			Object contentType = headers.get(CONTENT_TYPE);
-			String contentTypeOfPart = buildContentTypeAsString(headers, contentType);
+			String contentTypeOfPart = buildContentTypeAsString(contentType);
 
 			try {
 				//add headers
@@ -412,11 +457,11 @@ public class MimeMailBuilder {
 
 	private void appendHeadersToSimplePart(Part part, JSONObject headers) throws USMJSONAPIException {
 		try {
-			if (headers.has(MIME_VERSION))
+			if (headers != null && headers.has(MIME_VERSION))
 				part.setHeader(MIME_VERSION, getValue(headers, MIME_VERSION, 0, null));
 
-			if (headers.has(CONTENT_DISPOSITION)) {
-				String contDispAsString = buildContentDispositionAsString(headers, headers.get(CONTENT_DISPOSITION));
+			if (headers != null && headers.has(CONTENT_DISPOSITION)) {
+				String contDispAsString = buildContentDispositionAsString(headers.get(CONTENT_DISPOSITION));
 				part.setDisposition(contDispAsString);
 			}
 			part.setHeader(CONTENT_TRANSFER_ENCODING_UPPER, getValue(headers, CONTENT_TRANSFER_ENCODING, 0, null));
@@ -460,47 +505,85 @@ public class MimeMailBuilder {
 		knownAddressHeaderNamesList.add(DISPOSITION_NOTIFICATION_TO_ADDRESSLIST_HEADER);
 	}
 
+	private Enumeration<Header> parseXOriginalHeaders(String xOriginalHeaders) throws USMJSONAPIException {
+		ByteArrayInputStream is = new ByteArrayInputStream(Toolkit.decodeBase64(xOriginalHeaders));
+		try {
+			InternetHeaders ih = new InternetHeaders(is);
+			return ih.getAllHeaders();
+		} catch (MessagingException e) {
+			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_JAVAPI_MESSAGING_ERROR_PARSE_ORIGINAL_HEADERS,
+					ResponseStatusCode.INTERNAL_ERROR, "MessagingException by parsing x_original headers: " + e.getMessage());
+		}
+	}
+
 	private void appendHeadersToPart(Part part, JSONObject headers, String contentType) throws USMJSONAPIException {
 		try {
 
-			part.setHeader(CONTENT_TYPE_UPPER, contentType);
-			if (headers.has(CONTENT_DISPOSITION)) {
-				String contDispAsString = buildContentDispositionAsString(headers, headers.get(CONTENT_DISPOSITION));
+			if (contentType != null)
+				part.setHeader(CONTENT_TYPE_UPPER, contentType);
+			if (headers != null && headers.has(CONTENT_DISPOSITION)) {
+				String contDispAsString = buildContentDispositionAsString(headers.get(CONTENT_DISPOSITION));
 				part.setDisposition(contDispAsString);
 			}
 			appendHeaderConditional(part, CONTENT_TRANSFER_ENCODING_UPPER, getValue(headers, CONTENT_TRANSFER_ENCODING,
 					0, null));
-			String received = getValue(headers, RECEIVED, 1, null);
-			String received2 = getValue(headers, RECEIVED, 0, null);
+			//add (or replace with) the original headers
+			if (headers != null && headers.has(X_ORIGINAL_HEADERS)) {
+				Enumeration<Header> originalHeadersMap = parseXOriginalHeaders(headers.getString(X_ORIGINAL_HEADERS));
+				while (originalHeadersMap.hasMoreElements()) {
+					Header nextElement = originalHeadersMap.nextElement();
+					String headerName = nextElement.getName().toLowerCase();
+					if (RECEIVED.equalsIgnoreCase(headerName)) {
+						JSONArray receivedArray = headers.optJSONArray(RECEIVED);
+						if (receivedArray == null)
+							receivedArray = new JSONArray();
+						receivedArray.put(nextElement.getValue().replaceAll(terminalSeq, String.valueOf(SEMICOLON)));
+						headers.put(RECEIVED, receivedArray);
+					} else {
+						if (_isReplyOrForward) {
+							headers.put(headerName, nextElement.getValue());
+						} else {
+							if (!headers.has(headerName))
+								headers.put(headerName, nextElement.getValue());
+
+						}
+					}
+				}
+			} 
+			//add special headers
 			String returnPath = getValue(headers, RETURN_PATH, 0, null);
+		    String received = getValue(headers, RECEIVED, 0, null);
+			String received2 = getValue(headers, RECEIVED, 1, null);
 			String xSieve = getValue(headers, X_SIEVE, 0, null);
 
-			appendHeaderConditional(part, RETURN_PATH, returnPath);
+			setHeaderConditional(part, RETURN_PATH, returnPath);
 
-			if (headers.has(RECEIVED))
+			if (headers != null && headers.has(RECEIVED))
 				appendHeaderConditional(part, RECEIVED, formatReceived(received));
 
-			appendHeaderConditional(part, X_SIEVE, xSieve);
+			setHeaderConditional(part, X_SIEVE, xSieve);
 
-			if (headers.has(RECEIVED))
+			if (headers != null && headers.has(RECEIVED))
 				appendHeaderConditional(part, RECEIVED, formatReceived(received2));
 
-			if (headers.has(MIME_VERSION))
-				appendHeaderConditional(part, MIME_VERSION, getValue(headers, MIME_VERSION, 0, EMPTY_STR));
+			if (headers != null && headers.has(MIME_VERSION))
+				setHeaderConditional(part, MIME_VERSION, getValue(headers, MIME_VERSION, 0, EMPTY_STR));
 
-			if (headers.has(X_MAILER))
-				appendHeaderConditional(part, X_MAILER, getValue(headers, X_MAILER, 0, EMPTY_STR));
+			if (headers != null && headers.has(X_MAILER))
+				setHeaderConditional(part, X_MAILER, getValue(headers, X_MAILER, 0, EMPTY_STR));
 
 			//add additional headers
-			Iterable<String> headerNames = JSONToolkit.keys(headers);
-			for (String headerName : headerNames) {
-				if (knownAddressHeaderNamesList.contains(headerName)) {
-					appendHeaderConditional(part, headerName, getAddress(headers, headerName, null));
-				} else if (!specialHeaderNamesList.contains(headerName)) {
-					appendHeaderConditional(part, headerName, getValue(headers, headerName, 0, null));
+			if (headers != null) {
+				Iterable<String> headerNames = JSONToolkit.keys(headers);
+				for (String headerName : headerNames) {
+					if (knownAddressHeaderNamesList.contains(headerName)) {
+						setHeaderConditional(part, headerName, getAddress(headers, headerName, null));
+					} else if (!specialHeaderNamesList.contains(headerName)) {
+						setHeaderConditional(part, headerName, getValue(headers, headerName, 0, null));
+					}
 				}
 			}
-
+			
 		} catch (JSONException e) {
 			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_ADD_HEADER_JSON_ERROR,
 					ResponseStatusCode.WRONG_MISSING_PARAMETERS, "JSONException by add header: " + e.getMessage());
@@ -509,10 +592,11 @@ public class MimeMailBuilder {
 			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_ADD_HEADER_JAVAPI_MESSAGING_ERROR,
 					ResponseStatusCode.INTERNAL_ERROR, "MessagingException by add header: " + e.getMessage());
 		}
+
 	}
 
 	private String getValue(JSONObject headers, String key, int i, String defaultVal) throws JSONException {
-		if (!headers.has(key))
+		if (headers == null || !headers.has(key))
 			return defaultVal;
 		Object value = headers.get(key);
 		if (value == null)
@@ -537,43 +621,65 @@ public class MimeMailBuilder {
 
 	private String getAddress(JSONObject headers, String key, String defaultVal) throws JSONException,
 			USMJSONAPIException {
-		if (!headers.has(key))
+		if (headers == null || !headers.has(key))
 			return defaultVal;
 		Object value = headers.get(key);
 		if (value == null)
 			return defaultVal;
 		if (value instanceof String)
 			return value.toString();
-		else if (value instanceof JSONArray) {
-			JSONArray array = (JSONArray) value;
-			if (0 >= array.length())
-				return defaultVal;
-			try {
-				StringBuilder address = new StringBuilder();
-				for (int j = 0; j < array.length(); j++) {
-					String addressPart = array.getJSONObject(j).getString(ADDRESS);
-					String personalPart = array.getJSONObject(j).optString(PERSONAL, null);
-					QuotedInternetAddress ia = new QuotedInternetAddress(addressPart, personalPart, UTF_8);
-					address.append(ia.toString());
-					if (j < array.length() - 1)
-						address.append(",");
+		try {
+			if (value instanceof JSONObject)
+				return extractAddress((JSONObject) value);
+			if (value instanceof JSONArray) {
+				JSONArray array = (JSONArray) value;
+				int length = array.length();
+				if (0 >= length)
+					return defaultVal;
+				StringBuilder address = new StringBuilder(128);
+				for (int j = 0; j < length; j++) {
+					if (j > 0)
+						address.append(',');
+					address.append(extractAddress(array.getJSONObject(j)));
 				}
 				return address.toString();
-			} catch (JSONException e) {
-				return defaultVal;
-			} catch (AddressException e) {
-				throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_ADDRESS_EXCEPTION,
-						ResponseStatusCode.INTERNAL_ERROR, "mail address exception: " + e.getMessage());
-			} catch (UnsupportedEncodingException e) {
-				throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR_2,
-						ResponseStatusCode.INTERNAL_ERROR, "unsupported encoding of text content: " + e.getMessage());
 			}
-
+		} catch (JSONException e) {
+			return defaultVal;
+		} catch (UnsupportedEncodingException e) {
+			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR_2,
+					ResponseStatusCode.INTERNAL_ERROR, "unsupported encoding of text content: " + e.getMessage());
+		} catch (Exception e) {
+			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_ADDRESS_EXCEPTION,
+					ResponseStatusCode.INTERNAL_ERROR, "illegal mail address: " + e.getMessage());
 		}
 		return defaultVal;
 	}
 
+	private String extractAddress(JSONObject address) throws JSONException, UnsupportedEncodingException {
+		String addressPart = address.getString(ADDRESS);
+		String personalPart = address.optString(PERSONAL, null);
+		try {
+			return new QuotedInternetAddress(addressPart, personalPart, UTF_8).toString();
+		} catch (AddressException e) {
+			// In case of invalid address data, use simple composition of the raw (illegal) data
+			if (personalPart == null)
+				return addressPart;
+			return personalPart + " <" + addressPart + ">";
+		}
+	}
+
 	private void appendHeaderConditional(Part part, String header, String value) throws MessagingException {
+		if (value != null && !"null".equals(value)) {
+			try {
+				part.addHeader(header, MimeUtility.encodeText(value, UTF_8, null));
+			} catch (UnsupportedEncodingException e) {
+				part.addHeader(header, value);
+			}
+		}
+	}
+	
+	private void setHeaderConditional(Part part, String header, String value) throws MessagingException {
 		if (value != null && !"null".equals(value)) {
 			try {
 				part.setHeader(header, MimeUtility.encodeText(value, UTF_8, null));
@@ -587,37 +693,34 @@ public class MimeMailBuilder {
 		try {
 			if (dateObject.has("date"))
 				return dateObject.getString("date");
-			if (dateObject.has("utc")) {
-				//				DateFormat df = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG);
-				//				return df.format(new Date(dateObject.getLong("utc")));
-				return new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z").format(new Date(dateObject.getLong("utc")));
-			}
-		} catch (JSONException e) {
+			if (dateObject.has("utc"))
+				return _dateFormat.format(new Date(dateObject.getLong("utc")));
+		} catch (JSONException ignored) {
 		}
 		return null;
 	}
 
 	private String formatReceived(String rec) {
-		if (rec.length() < 2)
+		if (rec == null || rec.length() < 2)
 			return rec;
 		StringBuilder buffer = new StringBuilder(60);
-		int posBy = rec.indexOf("by");
-		int posFor = rec.indexOf("for");
-		if (posFor > 0) {
-
+		int posBy = rec.indexOf("by ");
+		if(posBy < 0) posBy = rec.indexOf("by" + terminalSeq);
+		int posFor = rec.indexOf("for ");
+		if (posFor > 0 && posBy > 0) {
 			buffer.append(rec.substring(0, posBy - 1)).append(terminalSeq);
-			buffer.append("	").append(rec.substring(posBy, posFor - 1)).append(terminalSeq);
-			buffer.append("	").append(rec.substring(posFor, rec.length()));
-		} else {
-			int posWith = rec.indexOf("with ");
+			buffer.append('	').append(rec.substring(posBy, posFor - 1)).append(terminalSeq);
+			buffer.append('	').append(rec.substring(posFor, rec.length()));
+		} else if (posBy > 0) {
+			int posWith = rec.indexOf("with ", posBy);
 			if (posWith < 0)
 				return rec;
 			int pos = rec.indexOf(SEMICOLON, posWith);
 			if (pos < 0 || (pos + 1) >= rec.length())
 				return rec;
 			buffer.append(rec.substring(0, posBy - 1)).append(terminalSeq);
-			buffer.append("	").append(rec.substring(posBy, pos + 1)).append(terminalSeq);
-			buffer.append("	").append(rec.substring(pos + 2, rec.length()));
+			buffer.append('	').append(rec.substring(posBy, pos + 1)).append(terminalSeq);
+			buffer.append('	').append(rec.substring(pos + 2, rec.length()));
 		}
 		return buffer.toString();
 	}
