@@ -107,8 +107,10 @@ public class SimpleDataObjectTest extends TestCase {
 			for (i = 0; i < fields.length; i++) {
 				j = parseField(text, fields[i].getFieldName(), fieldValues[i], j);
 			}
-			if (j < text.length())
-				throw new IllegalArgumentException("Extra text after object");
+			if (j < text.length()) {
+				if (!text.substring(j - 1).equals(",data:null}"))
+					throw new IllegalArgumentException("Extra text after object");
+			}
 			SimpleDataObject object = new SimpleDataObject(new SimSession(), type);
 			object.setTimestamp(Long.parseLong(timeStamps[0]));
 
@@ -280,7 +282,7 @@ public class SimpleDataObjectTest extends TestCase {
 			object.setChangeState(ChangeState.MODIFIED);
 			fail("setChangeState to " + ChangeState.MODIFIED + " on " + ChangeState.UNMODIFIED
 					+ " DataObject did not fail");
-		} catch (IllegalArgumentException e) {
+		} catch (IllegalArgumentException ignored) {
 		}
 	}
 
@@ -480,7 +482,8 @@ public class SimpleDataObjectTest extends TestCase {
 			1.0,
 			0L,
 			0L,
-			0L };
+			0L,
+			"12" };
 	private final static Object[] TEST_DATA2 = {
 			"Test-Data2",
 			"Test-Data2",
@@ -489,7 +492,8 @@ public class SimpleDataObjectTest extends TestCase {
 			2.0,
 			1L,
 			1L,
-			1L };
+			1L,
+			"7" };
 
 	public void testIsFieldModifiedInt() {
 		SimpleDataObject object = createTestObject();
@@ -577,30 +581,33 @@ public class SimpleDataObjectTest extends TestCase {
 	public void testToString() {
 		SimpleDataObject object = createTestObject();
 		assertEquals(
-				"SimpleDataObject(SimSession,SimContentType,CREATED){Timestamp:0,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}",
+				"SimpleDataObject(SimSession,SimContentType,CREATED){Timestamp:0,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
 				object.toString());
 		object.setTimestamp(1L);
 		assertEquals(
-				"SimpleDataObject(SimSession,SimContentType,CREATED){Timestamp:0=>1,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}",
+				"SimpleDataObject(SimSession,SimContentType,CREATED){Timestamp:0=>1,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
 				object.toString());
 		object.commitChanges();
 		assertEquals(
-				"SimpleDataObject(SimSession,SimContentType,UNMODIFIED){Timestamp:1,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}",
+				"SimpleDataObject(SimSession,SimContentType,UNMODIFIED){Timestamp:1,id:null,folder_id:null,Field2:[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
 				object.toString());
 		object.setFieldContent(2, new String[] { "Entry1", "Entry2" });
 		assertEquals(
 				"SimpleDataObject(SimSession,SimContentType,MODIFIED){Timestamp:1,id:null,folder_id:null,Field2:[]=>["
 						+ '"' + "Entry1" + '"' + "," + '"' + "Entry2" + '"'
-						+ "],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}", object.toString());
+						+ "],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
+				object.toString());
 		object.commitChanges();
 		assertEquals(
 				"SimpleDataObject(SimSession,SimContentType,UNMODIFIED){Timestamp:1,id:null,folder_id:null,Field2:["
 						+ '"' + "Entry1" + '"' + "," + '"' + "Entry2" + '"'
-						+ "],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}", object.toString());
+						+ "],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
+				object.toString());
 		object.setFieldContent(2, null);
 		assertEquals("SimpleDataObject(SimSession,SimContentType,MODIFIED){Timestamp:1,id:null,folder_id:null,Field2:["
 				+ '"' + "Entry1" + '"' + "," + '"' + "Entry2" + '"'
-				+ "]=>[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null}", object.toString());
+				+ "]=>[],Field3:null,Field4:null,Field5:null,Field6:null,Field7:null,created_by:null,data:null}",
+				object.toString());
 		SimpleDataObject o2 = createTestDataObjectFromString(SESSION, CONTENT_TYPE, object.toString());
 		assertEquals(object.toString(), o2.toString());
 	}
