@@ -49,7 +49,15 @@
 package com.openexchange.usm.session.impl;
 
 import java.io.Serializable;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.BitSet;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.TimeZone;
+import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 
@@ -57,15 +65,41 @@ import org.apache.commons.logging.Log;
 import org.json.JSONException;
 import org.json.JSONObject;
 
-import com.openexchange.usm.api.contenttypes.*;
+import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
+import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.database.EncapsulatedConnection;
-import com.openexchange.usm.api.exceptions.*;
-import com.openexchange.usm.api.session.*;
+import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
+import com.openexchange.usm.api.exceptions.DeserializationFailedException;
+import com.openexchange.usm.api.exceptions.FolderNotFoundException;
+import com.openexchange.usm.api.exceptions.InternalUSMException;
+import com.openexchange.usm.api.exceptions.OXCommunicationException;
+import com.openexchange.usm.api.exceptions.TimestampMismatchException;
+import com.openexchange.usm.api.exceptions.USMException;
+import com.openexchange.usm.api.exceptions.USMSQLException;
+import com.openexchange.usm.api.session.ConflictResolution;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.DataObjectFilter;
+import com.openexchange.usm.api.session.Folder;
+import com.openexchange.usm.api.session.OXConnectionInformation;
+import com.openexchange.usm.api.session.ObjectChanges;
+import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.api.session.SessionChangeListener;
+import com.openexchange.usm.api.session.SessionInitializer;
+import com.openexchange.usm.api.session.SyncResult;
 import com.openexchange.usm.mapping.FolderIdMapping;
+import com.openexchange.usm.ox_json.JSONResult;
+import com.openexchange.usm.ox_json.JSONResultType;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
-import com.openexchange.usm.session.dataobject.FolderImpl;
-import com.openexchange.usm.session.sync.*;
+import com.openexchange.usm.session.sync.ContentSyncerSupport;
+import com.openexchange.usm.session.sync.FolderElementsStorage;
+import com.openexchange.usm.session.sync.FolderHierarchyStorage;
+import com.openexchange.usm.session.sync.OXDataCacheID;
+import com.openexchange.usm.util.JSONToolkit;
+import com.openexchange.usm.util.Toolkit;
 
 public class SessionImpl implements Session {
 
@@ -77,6 +111,10 @@ public class SessionImpl implements Session {
 
 	private final static int FOLDER_ID_LENGTH_LIMIT = 39;
 
+	private final static int DIRECT_PING_RESPONSE_LIMIT = 1;
+
+	private static final long START_DATE_LIMIT = -100000000L * 86400L;
+
 	private final PersistentSessionData _persistentData;
 
 	private final SessionManagerImpl _sessionManager;
@@ -87,26 +125,34 @@ public class SessionImpl implements Session {
 
 	private final String _device;
 
-	private final CopyOnWriteArrayList<SessionChangeListener> _changeListeners = new CopyOnWriteArrayList<SessionChangeListener>();
+	private final List<SessionChangeListener> _changeListeners = new CopyOnWriteArrayList<SessionChangeListener>();
 
 	private final Map<String, Long> _syncedFolderContent = new ConcurrentHashMap<String, Long>();
 
 	private final Map<String, Object> _customProperties = new ConcurrentHashMap<String, Object>();
 
+	private final Map<ContentType, Comparator<? super DataObject>> _contentTypeSorters = new ConcurrentHashMap<ContentType, Comparator<? super DataObject>>();
+
 	private final Object _waitObject = new Object(); // Used to wait on folder changes and to notify about folder changes
 
+	private final DataObjectStorage _dataObjectCache;
+
+	private final OXDataCacheID _folderHierarchyCacheID;
+
 	private int _contextID;
 
 	private int _uniqueSessionID;
 
 	private int _userIdentifier;
 
+	private String _defaultInboxId;
+
+	private String _defaultAddress;
+
 	private String _password;
 
 	private DataObjectFilter _syncServerObjectFilter;
 
-	private DataObjectStorage _dataObjectCache;
-
 	private TimeZone _userTimeZone;
 
 	private String[] _rootFolders = new String[0];
@@ -125,10 +171,18 @@ public class SessionImpl implements Session {
 
 	private DataObjectFilter _lastEmailFilter = null;
 
-	private Map<String, Folder> _dummyFolderMap = new ConcurrentHashMap<String, Folder>();
-
 	private long _lastAccessCheck = 0L;
 
+	private OXConnectionInformation _oxConnectionData;
+
+	private int _folderIdLengthLimit;
+
+	private String _clientIP;
+
+	private int _directPingResponses = 0;
+
+	private Map<String, String> _xHeaders = new HashMap<String, String>();
+
 	public SessionImpl(SessionManagerImpl sessionManager, String user, String password, String protocol, String device) {
 		_sessionManager = sessionManager;
 		_user = user;
@@ -137,6 +191,7 @@ public class SessionImpl implements Session {
 		_device = device;
 		_persistentData = new PersistentSessionData(this);
 		_dataObjectCache = new DataObjectStorage(this);
+		_folderHierarchyCacheID = new OXDataCacheID(this);
 	}
 
 	public void initialize(SessionInitializer initializer, JSONObject configuration) throws USMException {
@@ -157,8 +212,12 @@ public class SessionImpl implements Session {
 				_sessionManager.getJournal().error(
 						this + " Can't resolve TimeZone " + timeZone + ", using " + _userTimeZone.getID());
 		} catch (JSONException e) {
-			_sessionManager.getJournal().error(this + " No identifier found in initialization", e);
+			_sessionManager.getJournal().error(this + " No timezone found in initialization", e);
 		}
+
+		_defaultInboxId = JSONToolkit.getString(configuration, "modules", "mail", "defaultFolder", "inbox");
+		_defaultAddress = JSONToolkit.getString(configuration, "modules", "mail", "defaultaddress");
+
 		_persistentData.initialize(initializer, configuration);
 	}
 
@@ -198,6 +257,10 @@ public class SessionImpl implements Session {
 		return _userIdentifier;
 	}
 
+	public String getDefaultEmailAddress() {
+		return _defaultAddress;
+	}
+
 	public String getPersistentField(String name) {
 		return _persistentData.getField(name);
 	}
@@ -279,8 +342,13 @@ public class SessionImpl implements Session {
 			_syncedFolderContent.put(folderId, 0L);
 			return _sessionManager.getIncrementalSyncer().syncChangesWithServer(
 					createFolderElementsStorage(folder, filter, storeResult, timestamp),
-					getConflictResolution(conflictResolution), _sessionManager.getJournal(),
-					getCachedFolderElements(folderId, contentType, timestamp), elements, timestamp, resultLimit);
+					getConflictResolution(conflictResolution),
+					getCachedFolderElements(folderId, contentType, timestamp), elements, timestamp, resultLimit,
+					filter, _contentTypeSorters.get(contentType));
+		} catch (FolderNotFoundException e) {
+			markFolderHierarchyDirty();
+			_syncedFolderContent.put(folderId, timestamp);
+			throw e;
 		} catch (USMException e) {
 			_syncedFolderContent.remove(folderId);
 			throw e;
@@ -290,7 +358,8 @@ public class SessionImpl implements Session {
 	public SyncResult syncWithServer(String folderId, int limit, DataObjectFilter filter, boolean storeResult,
 			ConflictResolution conflictResolution, DataObject... elements) throws USMException {
 		Folder folder = findFolder(folderId);
-		if (folder.getElementsContentType() == null)
+		ContentType contentType = folder.getElementsContentType();
+		if (contentType == null)
 			return SyncResult.EMPTY_RESULT;
 		if (_sessionManager.isEmailPullEnabled()
 				&& DefaultContentTypes.MAIL_ID.equals(folder.getElementsContentTypeID()))
@@ -299,7 +368,12 @@ public class SessionImpl implements Session {
 			_syncedFolderContent.put(folderId, 0L);
 			return _sessionManager.getSlowSyncer().syncWithServer(
 					createFolderElementsStorage(folder, filter, storeResult, 0L),
-					getConflictResolution(conflictResolution), _sessionManager.getJournal(), limit, elements);
+					getConflictResolution(conflictResolution), limit, filter, _contentTypeSorters.get(contentType),
+					elements);
+		} catch (FolderNotFoundException e) {
+			markFolderHierarchyDirty();
+			_syncedFolderContent.put(folderId, System.currentTimeMillis());
+			throw e;
 		} catch (USMException e) {
 			_syncedFolderContent.remove(folderId);
 			throw e;
@@ -327,18 +401,22 @@ public class SessionImpl implements Session {
 			boolean storeResult, ConflictResolution conflictResolution, Folder... folderChanges) throws USMException {
 		SyncResult result = _sessionManager.getIncrementalSyncer().syncChangesWithServer(
 				createFolderHierachyStorage(filter, storeResult, timestamp), getConflictResolution(conflictResolution),
-				_sessionManager.getJournal(), getCachedFolders(timestamp), folderChanges, timestamp, resultLimit);
+				getCachedFolders(timestamp), folderChanges, timestamp, resultLimit, filter, getFolderSorter());
 		return updateSynchronizationStateOfFolders(result);
 	}
 
 	public SyncResult syncFoldersWithServer(int limit, DataObjectFilter filter, boolean storeResult,
 			ConflictResolution conflictResolution, Folder... clientData) throws USMException {
 		SyncResult result = _sessionManager.getSlowSyncer().syncWithServer(
-				createFolderHierachyStorage(filter, storeResult, 0L), getConflictResolution(conflictResolution),
-				_sessionManager.getJournal(), limit, clientData);
+				createFolderHierachyStorage(filter, storeResult, 0L), getConflictResolution(conflictResolution), limit,
+				filter, getFolderSorter(), clientData);
 		return updateSynchronizationStateOfFolders(result);
 	}
 
+	private Comparator<? super DataObject> getFolderSorter() {
+		return _contentTypeSorters.get(_sessionManager.getFolderContentType());
+	}
+
 	private SyncResult updateSynchronizationStateOfFolders(SyncResult result) {
 		if (MARK_CHANGED_FOLDERS_AS_NEEDS_SYNCHRONIZATION) {
 			for (DataObject object : result.getChanges()) {
@@ -354,6 +432,10 @@ public class SessionImpl implements Session {
 		return result;
 	}
 
+	private void markFolderHierarchyDirty() {
+		_lastFolderChange = System.currentTimeMillis();
+	}
+
 	private FolderHierarchyStorage createFolderHierachyStorage(DataObjectFilter filter, boolean storeResult,
 			long oldTimestamp) {
 		return new FolderHierarchyStorage(this, filter == null ? _syncServerObjectFilter : filter, storeResult,
@@ -361,7 +443,7 @@ public class SessionImpl implements Session {
 	}
 
 	public Folder[] getCachedFolders() throws DatabaseAccessException, USMSQLException {
-		return getCachedFolders(_dataObjectCache.get(FOLDER_HIERARCHY_ID), 0L);
+		return getCachedFolders(_dataObjectCache.getNewestTimestamp(FOLDER_HIERARCHY_ID));
 	}
 
 	public Folder[] getCachedFolders(long timestamp) throws DatabaseAccessException, USMSQLException {
@@ -391,11 +473,15 @@ public class SessionImpl implements Session {
 	}
 
 	public Folder getCachedFolder(String folderID) throws DatabaseAccessException, USMSQLException {
-		return getCachedFolder(_dataObjectCache.get(FOLDER_HIERARCHY_ID), folderID, 0L);
+		Folder f = _sessionManager.getSpecialFolder(this, folderID);
+		if (f != null)
+			return f;
+		long newestTS = _dataObjectCache.getNewestTimestamp(FOLDER_HIERARCHY_ID);
+		return getCachedFolder(_dataObjectCache.get(FOLDER_HIERARCHY_ID, newestTS), folderID, newestTS);
 	}
 
 	private Folder getCachedFolder(Serializable[][] values, String folderID, long timestamp) {
-		if (values != null) {
+		if (values != null && folderID != null) {
 			Folder result = _sessionManager.getFolderContentType().newDataObject(this);
 			for (int i = 0; i < values.length; i++) {
 				try {
@@ -425,7 +511,9 @@ public class SessionImpl implements Session {
 
 	public DataObject[] getCachedFolderElements(String folderId, ContentType contentType)
 			throws DatabaseAccessException, USMSQLException {
-		return getCachedFolderElements(folderId, contentType, 0L, _dataObjectCache.get(getShortFolderID(folderId)));
+		String shortID = getShortFolderID(folderId);
+		long newestTS = _dataObjectCache.getNewestTimestamp(shortID);
+		return getCachedFolderElements(folderId, contentType, newestTS, _dataObjectCache.get(shortID, newestTS));
 	}
 
 	private DataObject[] getCachedFolderElements(String folderId, ContentType contentType, long timestamp,
@@ -434,6 +522,7 @@ public class SessionImpl implements Session {
 			return EMPTY_DATA_OBJECT_ARRAY;
 		if (values == null)
 			return null;
+		long start = System.currentTimeMillis();
 		DataObject[] result = new DataObject[values.length];
 		for (int i = 0; i < result.length; i++) {
 			DataObject o = contentType.newDataObject(this);
@@ -445,6 +534,11 @@ public class SessionImpl implements Session {
 			}
 			result[i] = o;
 		}
+		long end = System.currentTimeMillis();
+		if (_sessionManager.getJournal().isDebugEnabled())
+			_sessionManager.getJournal().debug(
+					this + " Deserialized " + result.length + " elements in " + folderId + " for " + timestamp + " in "
+							+ (end - start) + " ms.");
 		return result;
 	}
 
@@ -456,8 +550,9 @@ public class SessionImpl implements Session {
 
 	public DataObject getCachedFolderElement(String folderId, ContentType contentType, String objectID)
 			throws DatabaseAccessException, USMSQLException {
-		return getCachedFolderElement(folderId, contentType, objectID,
-				_dataObjectCache.get(getShortFolderID(folderId)), 0L);
+		String shortID = getShortFolderID(folderId);
+		long ts = _dataObjectCache.getNewestTimestamp(shortID);
+		return getCachedFolderElement(folderId, contentType, objectID, _dataObjectCache.get(shortID, ts), ts);
 	}
 
 	private DataObject getCachedFolderElement(String folderId, ContentType contentType, String objectID,
@@ -483,21 +578,19 @@ public class SessionImpl implements Session {
 		long result = internalStoreCachedDataObjects(FOLDER_HIERARCHY_ID, oldTimestamp, timestamp, objects);
 		if (!updatesPending)
 			_lastFolderSync = result;
-		int size = objects.size();
-		String[] cacheIDs = new String[size + 1 + _dummyFolderMap.size()];
-		String[] ids = new String[objects.size()];
-		int i = 0;
+		List<String> folderIDs = new ArrayList<String>();
+		List<String> shortIDs = new ArrayList<String>();
 		for (DataObject o : objects) {
-			cacheIDs[i] = getShortFolderID(o.getID());
-			ids[i++] = o.getID();
+			String id = o.getID();
+			folderIDs.add(id);
+			shortIDs.add(getShortFolderID(id));
 		}
-		cacheIDs[i++] = FOLDER_HIERARCHY_ID;
-		for (String folderID : _dummyFolderMap.keySet()) {
-			cacheIDs[i++] = getShortFolderID(folderID);
-		}
-		_dataObjectCache.retain(cacheIDs);
+		shortIDs.add(FOLDER_HIERARCHY_ID);
+		for (String specialFolder : _sessionManager.getSpecialFolders())
+			shortIDs.add(getShortFolderID(specialFolder));
+		_dataObjectCache.retain(shortIDs.toArray(new String[shortIDs.size()]));
 		if (_folderIdMapping != null)
-			_folderIdMapping.retain(ids);
+			_folderIdMapping.retain(folderIDs.toArray(new String[folderIDs.size()]));
 		return result;
 	}
 
@@ -515,7 +608,7 @@ public class SessionImpl implements Session {
 			DataObjectSet objects) throws DatabaseAccessException, USMSQLException {
 		if (!_sessionManager.isKeepLastSentSyncStateInDB())
 			oldTimestamp = 0L;
-		long resultingTimestamp = _dataObjectCache.put(folderID, timestamp, serializeObjects(objects), oldTimestamp);
+		long resultingTimestamp = _dataObjectCache.put(folderID, timestamp, objects, oldTimestamp, this);
 		if (resultingTimestamp != timestamp) {
 			for (DataObject o : objects) {
 				o.setTimestamp(resultingTimestamp);
@@ -534,14 +627,6 @@ public class SessionImpl implements Session {
 		return readDataObject(findFolder(folderID), objectID, fields);
 	}
 
-	private Serializable[][] serializeObjects(DataObjectSet objects) {
-		Serializable[][] data = new Serializable[objects.size()][];
-		int i = 0;
-		for (DataObject o : objects)
-			data[i++] = o.serialize();
-		return data;
-	}
-
 	private Folder completeStructure(Folder f) {
 		f.setElementsContentType(_sessionManager.getContentTypeManager().getContentType(f.getElementsContentTypeID()));
 		return f;
@@ -575,6 +660,7 @@ public class SessionImpl implements Session {
 			DataObjectSet newFolderHierarchy = new DataObjectSet();
 			long newTimestamp = checkTimestampAndUpdate(newFolderHierarchy, folderChanges, timestamp,
 					getCachedFolders(timestamp), "All Folders must have the same timestamp", resultMap);
+			_sessionManager.getOXDataCache().removeCachedData(_folderHierarchyCacheID);
 			if (newTimestamp != 0) {
 				long modifiedTimestamp = storeCachedFolders(timestamp, newTimestamp, newFolderHierarchy, true);
 				for (DataObject folder : folderChanges)
@@ -593,6 +679,7 @@ public class SessionImpl implements Session {
 				if (o.getParentFolderID().equals(folderId))
 					currentFolderChanges.add(elementChanges.remove(i));
 			}
+			_sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, folderId));
 			DataObjectSet newElements = new DataObjectSet();
 			long newTimestamp = checkTimestampAndUpdate(newElements, currentFolderChanges, timestamp,
 					getCachedFolderElements(folderId, first.getContentType(), timestamp),
@@ -721,7 +808,7 @@ public class SessionImpl implements Session {
 	}
 
 	public Folder findFolder(String folderID) throws USMException {
-		Folder folder = _dummyFolderMap.get(folderID);
+		Folder folder = _sessionManager.getSpecialFolder(this, folderID);
 		if (folder != null)
 			return folder;
 		folder = getCachedFolder(folderID);
@@ -730,12 +817,7 @@ public class SessionImpl implements Session {
 		FolderContentType type = _sessionManager.getFolderContentType();
 		Folder result = type.newDataObject(this);
 		result.setID(folderID);
-		try {
-			type.getTransferHandler().readDataObject(result, getFieldFilter(type));
-		} catch (USMException e) {
-			throw new FolderNotFoundException(USMSessionManagementErrorCodes.FOLDER_NOT_FOUND, "Folder not found: "
-					+ folderID, e);
-		}
+		type.getTransferHandler().readDataObject(result, getFieldFilter(type));
 		result.setElementsContentType(_sessionManager.getContentTypeManager().getContentType(
 				result.getElementsContentTypeID()));
 		return result;
@@ -750,6 +832,7 @@ public class SessionImpl implements Session {
 	}
 
 	public void foldersChanged(long timestamp) {
+		_sessionManager.getOXDataCache().removeCachedData(_folderHierarchyCacheID);
 		if (_changeListeners.size() > 0) {
 			ObjectChanges changes = new ObjectChangesImpl(true);
 			for (SessionChangeListener listener : _changeListeners) {
@@ -764,6 +847,7 @@ public class SessionImpl implements Session {
 			}
 		}
 		_lastFolderChange = timestamp;
+		_sessionManager.getJournal().debug("lastFolderChange " + _lastFolderChange);
 		synchronized (_waitObject) {
 			_waitObject.notifyAll();
 		}
@@ -780,6 +864,9 @@ public class SessionImpl implements Session {
 	}
 
 	public void folderContentChanged(long timestamp, String... folderIds) {
+		Log journal = _sessionManager.getJournal();
+		if (journal.isDebugEnabled())
+			journal.debug("Folder Content changed, timestamp: " + timestamp);
 		if (_changeListeners.size() > 0) {
 			ObjectChanges changes = new ObjectChangesImpl(false, folderIds);
 			for (SessionChangeListener listener : _changeListeners) {
@@ -792,13 +879,28 @@ public class SessionImpl implements Session {
 				}
 			}
 		}
-		for (String id : folderIds)
+		for (String id : folderIds) {
+			journal.debug("Removing folder from sync folder content, " + id);
 			_syncedFolderContent.remove(id);
+			_sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, id));
+			if ("1".equals(id)) { // for Remote Events when folderID == 1, it means that the default inbox has been changed
+				if (journal.isDebugEnabled())
+					journal.debug("Removing default inbox from sync folder content, " + _defaultInboxId);
+				_syncedFolderContent.remove(_defaultInboxId);
+				_sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, _defaultInboxId));
+			}
+		}
 		synchronized (_waitObject) {
 			_waitObject.notifyAll();
 		}
 	}
 
+	public boolean needsSynchronization(String folderID) {
+		if (Toolkit.provided(folderID))
+			return !_syncedFolderContent.containsKey(folderID);
+		return _lastFolderSync == 0L || _lastFolderSync < _lastFolderChange;
+	}
+
 	public ObjectChanges waitForChanges(boolean watchFolderChanges, String[] contentChangeParentIDsToWatch, long timeout)
 			throws USMException, InterruptedException {
 		Log journal = _sessionManager.getJournal();
@@ -824,9 +926,20 @@ public class SessionImpl implements Session {
 		}
 		synchronized (_waitObject) {
 			result = extractFolderChanges(watchFolderChanges, contentChangeParentIDsToWatch);
-			if (result != null)
+			if (result != null) {
+				if (++_directPingResponses > DIRECT_PING_RESPONSE_LIMIT && "EAS".equals(_protocol)) { //fix for bug 22741 - enable the ping response limit ONLY for EAS protocol
+					if (journal.isDebugEnabled())
+						_sessionManager.getJournal().debug(
+								this + " Too many pings without sync, marked as synchronized: "
+										+ Arrays.toString(contentChangeParentIDsToWatch));
+					for (String id : contentChangeParentIDsToWatch) {
+						_syncedFolderContent.put(id, 0L);
+					}
+				}
 				return result;
+			}
 		}
+		_directPingResponses = 0;
 		emailPullEnabled &= contentChangeParentIDsToWatch.length > 0;
 		long start = System.currentTimeMillis();
 		long end = start + timeout;
@@ -997,7 +1110,7 @@ public class SessionImpl implements Session {
 	}
 
 	public void setStartDate(long date) {
-		_startDate = date;
+		_startDate = (date < START_DATE_LIMIT) ? START_DATE_LIMIT : date;
 	}
 
 	public int getMailLimit() {
@@ -1014,7 +1127,7 @@ public class SessionImpl implements Session {
 				case 'D':
 					try {
 						return getFolderIdMapping().getLongID(Integer.parseInt(shortFolderID.substring(1)));
-					} catch (NumberFormatException e) {
+					} catch (NumberFormatException ignored) {
 					}
 					break;
 				case 'F':
@@ -1027,7 +1140,7 @@ public class SessionImpl implements Session {
 	}
 
 	public String getShortFolderID(String folderID) throws DatabaseAccessException, USMSQLException {
-		if (folderID.length() < FOLDER_ID_LENGTH_LIMIT)
+		if (folderID.length() < getFolderIdLengthLimit())
 			return "F" + folderID;
 		return "D" + getFolderIdMapping().getShortID(folderID);
 	}
@@ -1056,17 +1169,29 @@ public class SessionImpl implements Session {
 				throw new IllegalArgumentException("DataObjects of different folders");
 			set.add(o);
 		}
+		_sessionManager.getOXDataCache().removeCachedData(new OXDataCacheID(this, folderID));
 		return storeCachedDataObjects(folderID, timestampToKeep, timestamp, set, false);
 	}
 
+	public long storeSyncState(long timestampToKeep, long timestamp, DataObject[] objects) throws USMException {
+		for (DataObject o : objects) {
+			if (o.getContentType().getCode() != DefaultContentTypes.FOLDER_CODE)
+				throw new IllegalArgumentException("DataObjects are not folders");
+		}
+		_sessionManager.getOXDataCache().removeCachedData(_folderHierarchyCacheID);
+		DataObjectSet set = new DataObjectSet(objects);
+		return storeCachedFolders(timestampToKeep, timestamp, set, false);
+	}
+
 	public void endSyncronization() throws USMSQLException, DatabaseAccessException {
 		_dataObjectCache.removeAllObjectsForSession();
 		_persistentData.removeAllDBFieldsForSession();
 		_sessionManager.removeSessionFromStorage(this);
 	}
 
-	public void storeUUID(ContentType contentType, int objectId, UUID uuid) throws USMException {
-		_sessionManager.storeUUID(_contextID, contentType, objectId, uuid);
+	public void storeUUID(ContentType contentType, int objectId, UUID uuid, int folderOwner) throws USMException {
+		_sessionManager.storeUUID(_contextID, contentType, objectId, uuid, (folderOwner == _userIdentifier) ? 0
+				: folderOwner);
 	}
 
 	public PersistentSessionData getPersistentSessionData() {
@@ -1090,16 +1215,7 @@ public class SessionImpl implements Session {
 	}
 
 	public Folder getDummyContentTypeFolder(ContentType type) {
-		Folder folder = _dummyFolderMap.get(type.getID());
-		if (folder != null)
-			return folder;
-		folder = new FolderImpl(this, _sessionManager.getFolderContentType());
-		folder.setID(type.getID());
-		long timestamp = System.currentTimeMillis();
-		folder.setTimestamp(timestamp);
-		folder.setElementsContentType(type);
-		_dummyFolderMap.put(type.getID(), folder);
-		return folder;
+		return _sessionManager.getSpecialFolder(this, type.getID());
 	}
 
 	public ContentType[] getContentTypes() throws DatabaseAccessException, USMSQLException {
@@ -1122,4 +1238,94 @@ public class SessionImpl implements Session {
 			OXCommunicationException {
 		return _sessionManager.getOXAjaxAccess().getConfiguration(this, path);
 	}
+
+	public boolean checkProperDBStorage() throws DatabaseAccessException, USMSQLException {
+		return _persistentData.checkProperDBStorage();
+	}
+
+	public void setContentTypeSorter(ContentType contentType, Comparator<? super DataObject> comparator) {
+		if (comparator == null)
+			_contentTypeSorters.remove(contentType);
+		else
+			_contentTypeSorters.put(contentType, comparator);
+	}
+
+	public OXConnectionInformation getOXConnectionData() {
+		return _oxConnectionData;
+	}
+
+	public void setOXConnectionData(OXConnectionInformation data) {
+		_oxConnectionData = data;
+	}
+
+	public OXDataCacheID getFolderHierarchyCacheID() {
+		return _folderHierarchyCacheID;
+	}
+
+	public int getFolderIdLengthLimit() {
+		return _folderIdLengthLimit > 0 ? _folderIdLengthLimit : FOLDER_ID_LENGTH_LIMIT;
+	}
+
+	public void setFolderIdLengthLimit(int limit) {
+		_folderIdLengthLimit = limit;
+	}
+
+	public String getClientIp() {
+		return _clientIP;
+	}
+
+	public void setClientIp(String ip) {
+		_clientIP = ip;
+	}
+
+	public void resetClientIp(String ip) {
+		if (_clientIP.equals(ip))
+			return;
+		_clientIP = ip;
+		Log journal = _sessionManager.getJournal();
+		Map<String, String> parameters = new HashMap<String, String>();
+		parameters.put("clientIP", _clientIP);
+		try {
+			JSONResult result = _sessionManager.getOXAjaxAccess().doPost("login", "changeip", this, parameters);
+			if (result.getResultType() == JSONResultType.Error && journal.isDebugEnabled())
+				journal.debug(this + " Changing ip failed: " + result.toString());
+		} catch (AuthenticationFailedException e) {
+			if (journal.isDebugEnabled())
+				journal.debug(this + " Authentication failed on changing client ip.");
+		} catch (OXCommunicationException e) {
+			if (journal.isDebugEnabled())
+				journal.debug(this + " OX Communication Exception on changing client ip.");
+		}
+	}
+
+	public Map<String, String> getXRequestHeaders() {
+		return _xHeaders;
+	}
+
+	public void setXRequestHeaders(Map<String, String> headers) {
+		_xHeaders = headers;
+	}
+
+	public long getNewestTimestamp(String folderID) {
+		Log journal = _sessionManager.getJournal();
+		try {
+			String shortID = getShortFolderID(folderID);
+			return _dataObjectCache.getNewestTimestamp(shortID);
+		} catch (DatabaseAccessException e) {
+			if (journal.isDebugEnabled())
+				journal
+						.debug(this + " DatabaseAccessException on getting the newest timestamp for folder: "
+								+ folderID);
+			return 0;
+		} catch (USMSQLException e) {
+			if (journal.isDebugEnabled())
+				journal.debug(this + " USMSQLException on getting the newest timestamp for folder: " + folderID);
+			return 0;
+		}
+	}
+
+	public void invalidateCachedData(String folderID) {
+		_sessionManager.getOXDataCache().removeCachedData(
+				Toolkit.provided(folderID) ? new OXDataCacheID(this, folderID) : _folderHierarchyCacheID);
+	}
 }
