@@ -48,28 +48,53 @@
  */
 package com.openexchange.usm.session.impl;
 
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 import java.util.UUID;
 
-import javax.management.*;
+import javax.management.MalformedObjectNameException;
+import javax.management.NotCompliantMBeanException;
+import javax.management.ObjectName;
 
 import org.apache.commons.logging.Log;
 import org.json.JSONObject;
 
 import com.openexchange.management.ManagementException;
 import com.openexchange.management.ManagementService;
+import com.openexchange.timer.ScheduledTimerTask;
+import com.openexchange.timer.TimerService;
 import com.openexchange.usm.api.USMVersion;
-import com.openexchange.usm.api.contenttypes.*;
+import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeManager;
+import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.database.DatabaseAccess;
 import com.openexchange.usm.api.database.DatabaseAccessException;
-import com.openexchange.usm.api.exceptions.*;
-import com.openexchange.usm.api.session.*;
-import com.openexchange.usm.configuration.*;
+import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
+import com.openexchange.usm.api.exceptions.OXCommunicationException;
+import com.openexchange.usm.api.exceptions.USMAccessDeniedException;
+import com.openexchange.usm.api.exceptions.USMException;
+import com.openexchange.usm.api.exceptions.USMSQLException;
+import com.openexchange.usm.api.exceptions.USMStartupException;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.Folder;
+import com.openexchange.usm.api.session.Session;
+import com.openexchange.usm.api.session.SessionInitializer;
+import com.openexchange.usm.api.session.SessionManager;
+import com.openexchange.usm.configuration.ConfigurationManager;
+import com.openexchange.usm.configuration.ConfigurationProperties;
+import com.openexchange.usm.configuration.USMInvalidConfigurationException;
 import com.openexchange.usm.mapping.ObjectIdMappingService;
 import com.openexchange.usm.ox_event.OXEventListener;
 import com.openexchange.usm.ox_event.OXEventManager;
 import com.openexchange.usm.ox_json.OXJSONAccess;
 import com.openexchange.usm.session.dataobject.DataObjectSet;
+import com.openexchange.usm.session.dataobject.FolderImpl;
 import com.openexchange.usm.session.sync.IncrementalContentSyncer;
+import com.openexchange.usm.session.sync.OXDataCache;
 import com.openexchange.usm.session.sync.SlowContentSyncer;
 import com.openexchange.usm.util.JSONToolkit;
 import com.openexchange.usm.uuid.OXObjectID;
@@ -82,6 +107,18 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 	// LATER we might want to make this constant configurable
 	private final static long ACCESS_CHECK_INTERVAL = 300000L;
 
+	private final static long ONE_DAY_IN_MILLISECONDS = 24L * 60L * 60L * 1000L;
+
+	public static final String LAST_ACCESS_FIELD_NAME = "USM_ACCESS";
+
+	private final static List<String> SPECIAL_FOLDERS = new ArrayList<String>();
+	private final static List<String> SPECIAL_FOLDERS_VIEW = Collections.unmodifiableList(SPECIAL_FOLDERS);
+
+	static {
+		SPECIAL_FOLDERS.add(DefaultContentTypes.GROUPS_ID);
+		SPECIAL_FOLDERS.add(DefaultContentTypes.RESOURCES_ID);
+	}
+
 	private OXJSONAccess _oxAjaxAccess;
 
 	private Log _journal;
@@ -92,6 +129,8 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 
 	private SessionStorage _sessionStorage;
 
+	private OXDataCache _dataCache;
+
 	private OXEventManager _oxEventManager;
 
 	private ConfigurationManager _configurationManager;
@@ -117,9 +156,31 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 
 	private boolean _keepLastSentSyncStateInDB = ConfigurationProperties.SYNC_KEEP_LAST_SENT_STATE_IN_DB_DEFAULT;
 
-	private SlowContentSyncer _slowSyncer;
+	private long _sessionStorageTimeLimit = ConfigurationProperties.USM_SESSION_STORAGE_TIME_LIMIT_DEFAULT
+			* ONE_DAY_IN_MILLISECONDS;
 
-	private IncrementalContentSyncer _incrementalSyncer;
+	private TimerService _timerService;
+
+	private ScheduledTimerTask _cleanupTask;
+
+	private int _syncRetryCount;
+
+	private boolean _ignoreClientDataAtSlowSync;
+
+	private boolean _forceNewSyncKeyOnEmptySync;
+
+	public SessionManagerImpl() {
+	}
+
+	/**
+	 * This constructor is used only for tests of the IncrementalContentSyncer and SlowContentSyncer.
+	 * 
+	 * @param log
+	 */
+	public SessionManagerImpl(Log log) {
+		_journal = log;
+		_dataCache = new OXDataCache(0, 0);
+	}
 
 	public void activate(Log log, OXJSONAccess ajaxAccess, ContentTypeManager contentTypeManager,
 			FolderContentType folderContentType, OXEventManager oxEventManager,
@@ -134,17 +195,14 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 		_databaseAccess = dbAccess;
 		_mappingService = mappingService;
 		_uuidService = uuidService;
-		int retryCount;
 		try {
-			retryCount = _configurationManager.getProperty(
+			_syncRetryCount = _configurationManager.getProperty(
 					ConfigurationProperties.SYNC_CONCURRENT_MODIFICATION_MAX_RETRIES_PROPERTY,
 					ConfigurationProperties.SYNC_CONCURRENT_MODIFICATION_MAX_RETRIES_DEFAULT, false);
 		} catch (USMInvalidConfigurationException e) {
 			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR,
 					"Configuration for Sync retries is invalid", e);
 		}
-		_slowSyncer = new SlowContentSyncer(this, retryCount);
-		_incrementalSyncer = new IncrementalContentSyncer(this, retryCount);
 		_waitForChangesEmailPullDelay = readEmailPullDelayProperty();
 		_waitForChangesEmailMinPullDelay = readEmailMinPullDelayProperty();
 		try {
@@ -175,7 +233,55 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 					"Configuration for usm interface module is invalid", e);
 		}
 
-		_sessionStorage = new SessionStorage();
+		try {
+			_sessionStorageTimeLimit = _configurationManager.getProperty(
+					ConfigurationProperties.USM_SESSION_STORAGE_TIME_LIMIT_PROPERTY,
+					ConfigurationProperties.USM_SESSION_STORAGE_TIME_LIMIT_DEFAULT, false)
+					* ONE_DAY_IN_MILLISECONDS;
+		} catch (USMInvalidConfigurationException e) {
+			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR_9,
+					"Invalid configuration for " + ConfigurationProperties.USM_SESSION_STORAGE_TIME_LIMIT_PROPERTY, e);
+		}
+
+		int cacheTimeLimitSyncComplete;
+		int cacheTimeLimitSyncIncomplete;
+		try {
+			cacheTimeLimitSyncComplete = _configurationManager.getProperty(
+					ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_COMPLETE_PROPERTY,
+					ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_COMPLETE_DEFAULT, false);
+		} catch (USMInvalidConfigurationException e) {
+			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR_10,
+					"Invalid configuration for " + ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_COMPLETE_PROPERTY, e);
+		}
+		try {
+			cacheTimeLimitSyncIncomplete = _configurationManager.getProperty(
+					ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_INCOMPLETE_PROPERTY,
+					ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_INCOMPLETE_DEFAULT, false);
+		} catch (USMInvalidConfigurationException e) {
+			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR_11,
+					"Invalid configuration for " + ConfigurationProperties.SYNC_CACHE_TIME_LIMIT_INCOMPLETE_PROPERTY, e);
+		}
+
+		try {
+			_ignoreClientDataAtSlowSync = _configurationManager.getProperty(
+					ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC,
+					ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC_DEFAULT, false);
+		} catch (USMInvalidConfigurationException e) {
+			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR_12,
+					"Invalid configuration for " + ConfigurationProperties.USM_IGNORE_CLIENT_DATA_AT_SLOW_SYNC, e);
+		}
+
+		try {
+			_forceNewSyncKeyOnEmptySync = _configurationManager.getProperty(
+					ConfigurationProperties.USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC,
+					ConfigurationProperties.USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC_DEFAULT, false);
+		} catch (USMInvalidConfigurationException e) {
+			throw new USMStartupException(USMSessionManagementErrorCodes.CONFIGURATION_ERROR_13,
+					"Invalid configuration for " + ConfigurationProperties.USM_FORCE_NEW_SYNC_KEY_ON_EMPTY_SYNC, e);
+		}
+
+		_sessionStorage = new SessionStorage(this);
+		_dataCache = new OXDataCache(cacheTimeLimitSyncComplete, cacheTimeLimitSyncIncomplete);
 		_oxEventManager.addEventListener(this);
 
 		_journal.info("USM SessionManager activated");
@@ -192,6 +298,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 		_sessionStorage.shutdown(_oxAjaxAccess);
 		_configurationManager = null;
 		_oxEventManager = null;
+		_dataCache = null;
 		_sessionStorage = null;
 		_contentTypeManager = null;
 		_oxAjaxAccess = null;
@@ -199,40 +306,46 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 		_mappingService = null;
 		_waitForChangesEmailPullDelay = 0;
 		_waitForChangesEmailMinPullDelay = -1;
-		_slowSyncer = null;
-		_incrementalSyncer = null;
 		_journal.info("USM SessionManager deactivated");
 		_journal = null;
 	}
 
 	public Session getSession(String user, String password, String protocol, String device) throws USMException {
-		return getSession(user, password, protocol, device, null);
+		return getSession(user, password, protocol, device, null, "127.0.0.1", new HashMap<String, String>());
 	}
 
 	public Session getSession(String user, String password, String protocol, String device,
-			SessionInitializer initializer) throws USMException {
+			SessionInitializer initializer, String clientIP, Map<String, String> xHeaders) throws USMException {
 		SessionID id = new SessionID(user, protocol, device);
 		SessionImpl session = _sessionStorage.getSession(id);
 		if (session != null) {
-			if (!session.getPassword().equals(password)) {
-				session.setPassword(password);
-				_oxAjaxAccess.login(session);
-			}
-			try {
-				recheckUSMAccess(initializer, session);
-				return session;
-			} catch (USMAccessDeniedException e) {
-				_sessionStorage.removeSession(session);
-				throw e;
+			if (session.checkProperDBStorage()) {
+				if (!session.getPassword().equals(password)) {
+					session.setPassword(password);
+					session.setClientIp(clientIP);
+					_oxAjaxAccess.login(session);
+				}
+				try {
+					recheckUSMAccess(initializer, session);
+					return session;
+				} catch (USMAccessDeniedException e) {
+					_sessionStorage.removeSession(session);
+					throw e;
+				}
 			}
+			// Cached Session is no longer present in DB, start from scratch
+			_sessionStorage.removeSession(session);
 		}
 		session = new SessionImpl(this, user, password, protocol, device);
+		session.setClientIp(clientIP);
+		session.setXRequestHeaders(xHeaders);
 		_oxAjaxAccess.login(session);
 		JSONObject configuration = _oxAjaxAccess.getConfiguration(session);
 		checkUSMAccess(session, configuration);
 		session.setLastAccessCheck(System.currentTimeMillis());
 		session.initialize(initializer, configuration);
 		_sessionStorage.storeSession(id, session);
+		session.setPersistentField(LAST_ACCESS_FIELD_NAME, String.valueOf(System.currentTimeMillis()));
 		return session;
 	}
 
@@ -249,7 +362,7 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 			AuthenticationFailedException, OXCommunicationException {
 		if (_usmInterface == null || _usmInterface.length() == 0) {
 			JSONObject usmModule = getUSMModule(session, configuration, "com.openexchange.usm");
-			if (usmModule != null && usmModule.optBoolean("module"))
+			if (usmModule != null && usmModule.optBoolean("active"))
 				return;
 		} else {
 			JSONObject oxInterfaces = getUSMModule(session, configuration, "interfaces");
@@ -306,7 +419,10 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 
 	public void removeSessionFromStorage(SessionImpl session) {
 		_sessionStorage.removeSession(session);
-		_oxAjaxAccess.logout(session);
+	}
+
+	public boolean isIgnoreClientDataAtSlowSync() {
+		return _ignoreClientDataAtSlowSync;
 	}
 
 	// --- Interface OXEventListener ---
@@ -392,18 +508,21 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 		if (contentTypeCode != DefaultContentTypes.MAIL_CODE) {
 			try {
 				int objectId = Integer.parseInt(o.getID());
-				_uuidService.storeMapping(o.getUUID(), new OXObjectID(o.getSession().getContextId(), contentTypeCode,
-						objectId));
+				int folderOwner = o.getParentFolderOwnerID();
+				if (folderOwner == o.getSession().getUserIdentifier())
+					folderOwner = 0;
+				_uuidService.storeMapping(getModifedUUID(o.getUUID(), folderOwner), new OXObjectID(o.getSession()
+						.getContextId(), contentTypeCode, objectId));
 			} catch (NumberFormatException ignored) {
 				// For all cases in which we have no integer OX object id (e.g. mail folders), only store UUIDs locally in sync states
 			}
 		}
 	}
 
-	public void storeUUID(int cid, ContentType contentType, int objectId, UUID uuid) throws DatabaseAccessException,
-			USMSQLException, UUIDAlreadyMappedException, OXObjectAlreadyMappedException {
+	public void storeUUID(int cid, ContentType contentType, int objectId, UUID uuid, int folderOwner)
+			throws DatabaseAccessException, USMSQLException, UUIDAlreadyMappedException, OXObjectAlreadyMappedException {
 		OXObjectID oxObjectId = new OXObjectID(cid, contentType.getCode(), objectId);
-		_uuidService.storeMapping(uuid, oxObjectId);
+		_uuidService.storeMapping(getModifedUUID(uuid, folderOwner), oxObjectId);
 	}
 
 	//	public void storeOrRemapUUID(int cid, ContentType contentType, int objectId, UUID uuid)
@@ -414,10 +533,12 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 
 	public DataObjectSet insertStoredUUID(DataObject o, DataObjectSet oldData, DataObjectSet newestCachedData)
 			throws DatabaseAccessException, USMSQLException {
+		boolean useModifiedUUID = o.getSession().getUserIdentifier() != o.getParentFolderOwnerID();
 		if (oldData != null) {
 			DataObject oldObject = oldData.get(o.getID());
 			if (oldObject != null) {
-				o.setUUID(oldObject.getUUID());
+				o.setUUID(useModifiedUUID ? getModifedUUID(oldObject.getUUID(), o.getParentFolderOwnerID()) : oldObject
+						.getUUID());
 				return newestCachedData;
 			}
 		}
@@ -432,16 +553,19 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 				break;
 			case DefaultContentTypes.MAIL_CODE:
 				if (newestCachedData == null)
+					// TODO Optimize so that newest cached data is only retrieved if old data is not already newest cached data ?
 					newestCachedData = new DataObjectSet(o.getSession().getCachedFolderElements(o.getParentFolderID(),
 							o.getContentType()));
 				DataObject cachedMail = newestCachedData.get(o.getID());
 				if (cachedMail != null)
-					o.setUUID(cachedMail.getUUID());
+					o.setUUID(useModifiedUUID ? getModifedUUID(cachedMail.getUUID(), o.getParentFolderOwnerID())
+							: cachedMail.getUUID());
 				break;
 			default:
 				try {
 					int objectId = Integer.parseInt(o.getID());
-					o.setUUID(_uuidService.getUUID(o.getSession().getContextId(), contentTypeCode, objectId));
+					UUID uuid = _uuidService.getUUID(o.getSession().getContextId(), contentTypeCode, objectId);
+					o.setUUID(useModifiedUUID ? getModifedUUID(uuid, o.getParentFolderOwnerID()) : uuid);
 				} catch (NumberFormatException ignored) {
 				}
 				break;
@@ -546,10 +670,66 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
 	}
 
 	public SlowContentSyncer getSlowSyncer() {
-		return _slowSyncer;
+		return new SlowContentSyncer(this, _syncRetryCount);
 	}
 
 	public IncrementalContentSyncer getIncrementalSyncer() {
-		return _incrementalSyncer;
+		return new IncrementalContentSyncer(this, _syncRetryCount);
+	}
+
+	public void setTimerService(TimerService timerService) {
+		if (_cleanupTask != null) {
+			_cleanupTask.cancel();
+			_cleanupTask = null;
+		}
+		_timerService = timerService;
+		if (_timerService != null)
+			_cleanupTask = _timerService.scheduleAtFixedRate(new USMSessionCleanupTask(this),
+					USMSessionCleanupTask.INITIAL_DELAY, USMSessionCleanupTask.EXECUTION_INTERVAL);
+	}
+
+	public long getSessionStorageTimeLimit() {
+		return _sessionStorageTimeLimit;
+	}
+
+	/**
+	 * Creates a modified UUID from a base UUID and a given OX user id. The algorithm uses XOR, so this
+	 * method can be used both ways, i.e. given a modified UUID and OX user id, it reports back the
+	 * original unmodified UUID. If either the UUID is null or the userID is 0, this method returns the
+	 * unmodified UUID.
+	 * 
+	 * @param uuid
+	 * @param userID
+	 * @return
+	 */
+	public UUID getModifedUUID(UUID uuid, int userID) {
+		if (uuid == null || userID == 0)
+			return uuid;
+		return new UUID(uuid.getMostSignificantBits() ^ userID, uuid.getLeastSignificantBits());
+	}
+
+	public OXDataCache getOXDataCache() {
+		return _dataCache;
+	}
+
+	public Folder getSpecialFolder(SessionImpl session, String id) {
+		if (!DefaultContentTypes.GROUPS_ID.equals(id) && !DefaultContentTypes.RESOURCES_ID.equals(id))
+			return null;
+		ContentType type = getContentTypeManager().getContentType(id);
+		if (type == null)
+			return null;
+		Folder folder = new FolderImpl(session, getFolderContentType());
+		folder.setID(id);
+		folder.setTimestamp(System.currentTimeMillis());
+		folder.setElementsContentType(type);
+		return folder;
+	}
+
+	public List<String> getSpecialFolders() {
+		return SPECIAL_FOLDERS_VIEW;
+	}
+
+	public boolean isForceNewSyncKeyOnEmptySync() {
+		return _forceNewSyncKeyOnEmptySync;
 	}
 }
