@@ -51,20 +51,29 @@
  */
 package com.openexchange.usm.session.sync.test;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.UUID;
 
 import junit.framework.TestCase;
 
-import org.apache.commons.logging.Log;
-
 import com.openexchange.usm.api.exceptions.ConflictingChange;
 import com.openexchange.usm.api.exceptions.USMException;
-import com.openexchange.usm.api.session.*;
+import com.openexchange.usm.api.session.ChangeState;
+import com.openexchange.usm.api.session.ConflictResolution;
+import com.openexchange.usm.api.session.DataObject;
+import com.openexchange.usm.api.session.ProtocolInformation;
+import com.openexchange.usm.api.session.SimSession;
+import com.openexchange.usm.api.session.SyncResult;
+import com.openexchange.usm.api.session.SynchronizationConflictException;
 import com.openexchange.usm.configuration.ConfigurationProperties;
 import com.openexchange.usm.journal.SimJournal;
 import com.openexchange.usm.session.dataobject.DataObjectUtil;
 import com.openexchange.usm.session.impl.SessionManagerImpl;
-import com.openexchange.usm.session.sync.*;
+import com.openexchange.usm.session.sync.IncrementalContentSyncer;
+import com.openexchange.usm.session.sync.SimContentSyncerStorage;
+import com.openexchange.usm.session.sync.SimSyncContentType;
+import com.openexchange.usm.session.sync.SimSyncContentTypeTransferHandler;
 
 /**
  * @author ldo
@@ -75,6 +84,50 @@ public class IncrementalContentSyncerTest extends TestCase {
 		void run(ConflictResolution resolution) throws Exception;
 	}
 
+	private static class TestProtocolInfo implements ProtocolInformation {
+		private static final long serialVersionUID = 1L;
+
+		private final String _data;
+
+		public TestProtocolInfo(String data) {
+			_data = data;
+		}
+
+		public String getData() {
+			return _data;
+		}
+
+		@Override
+		public String toString() {
+			return "ProtocolInfo(" + _data + ')';
+		}
+
+		@Override
+		public int hashCode() {
+			final int prime = 31;
+			int result = 1;
+			result = prime * result + ((_data == null) ? 0 : _data.hashCode());
+			return result;
+		}
+
+		@Override
+		public boolean equals(Object obj) {
+			if (this == obj)
+				return true;
+			if (obj == null)
+				return false;
+			if (!(obj instanceof TestProtocolInfo))
+				return false;
+			TestProtocolInfo other = (TestProtocolInfo) obj;
+			if (_data == null) {
+				if (other._data != null)
+					return false;
+			} else if (!_data.equals(other._data))
+				return false;
+			return true;
+		}
+	}
+
 	private static final int INITIAL_SMALL_DATAOBJECT_COUNT = 2;
 	private static final int INITIAL_LARGE_DATAOBJECT_COUNT = 100;
 	private static final boolean WAIT_AT_END = false;
@@ -85,12 +138,11 @@ public class IncrementalContentSyncerTest extends TestCase {
 	List<DataObject> expectedClientChanges = new ArrayList<DataObject>();
 	List<DataObject> dataOnServer = new ArrayList<DataObject>();
 	SimContentSyncerStorage syncerStorage = new SimContentSyncerStorage();
-	SimJournal journal = new SimJournal();
-	Log log = journal.getLog(IncrementalContentSyncerTest.class);
 	SimSession session = new SimSession();
 	SimSyncContentType contentType = new SimSyncContentType();
 	SimSyncContentTypeTransferHandler handler = (SimSyncContentTypeTransferHandler) contentType.getTransferHandler();
-	IncrementalContentSyncer contentSyncer = new IncrementalContentSyncer(new SessionManagerImpl(),
+	IncrementalContentSyncer contentSyncer = new IncrementalContentSyncer(new SessionManagerImpl(new SimJournal()
+			.getLog(IncrementalContentSyncerTest.class)),
 			ConfigurationProperties.SYNC_CONCURRENT_MODIFICATION_MAX_RETRIES_DEFAULT);
 
 	public void testNoChangesOnClientAndServer() {
@@ -106,11 +158,13 @@ public class IncrementalContentSyncerTest extends TestCase {
 		performMultiCheck(new ResolutionRunnable() {
 			public void run(ConflictResolution resolution) throws Exception {
 				initOriginalState();
-				DataObject o = createDataObject("clientID", "Client", 1, false);
+				TestProtocolInfo protocolInfo = new TestProtocolInfo("ClientProtInfo");
+				DataObject o = createDataObject("clientID", "Client", protocolInfo, 1, false);
 				clientChanges.add(o);
 				DataObject o2 = o.createCopy(false);
 				o2.setTimestamp(2L);
 				o2.commitChanges();
+				o2.setProtocolInformation(protocolInfo);
 				expectedSyncState.add(o2);
 				handler._writeNewToCall = true;
 				handler._objectToSend = o;
@@ -123,7 +177,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 	public void testServerCreation() {
 		performMultiCheck(new ResolutionRunnable() {
 			public void run(ConflictResolution resolution) throws Exception {
-				initOriginalState(createDataObject("serverID", "Server", 1, true));
+				initOriginalState(createDataObject("serverID", "Server", null, 1, true));
 				performSync(resolution, 0);
 			}
 		}, ConflictResolution.values());
@@ -154,7 +208,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 				o.setTimestamp(2L);
 				o.setChangeState(ChangeState.DELETED);
 				expectedClientChanges.add(o);
-				expectedSyncState.remove(1);
+				o.setProtocolInformation(expectedSyncState.remove(1).getProtocolInformation());
 				performSync(resolution, 0);
 			}
 		}, ConflictResolution.values());
@@ -166,9 +220,11 @@ public class IncrementalContentSyncerTest extends TestCase {
 				initOriginalState();
 				DataObject o = syncState.get(1).createCopy(false);
 				o.setFieldContent("Field2", "ModifiedClient");
+				o.setProtocolInformation(syncState.get(1).getProtocolInformation());
 				clientChanges.add(o);
 				DataObject o2 = o.createCopy(false);
 				o2.setTimestamp(2L);
+				o2.setProtocolInformation(o.getProtocolInformation());
 				o2.commitChanges();
 				expectedSyncState.set(1, o2);
 				handler._writeUpdateToCall = true;
@@ -190,7 +246,10 @@ public class IncrementalContentSyncerTest extends TestCase {
 				o2.commitChanges();
 				o2.setTimestamp(2L);
 				o2.setFieldContent("Field2", "ModifiedServer");
-				expectedClientChanges.add(o2.createCopy(false));
+				o2.setProtocolInformation(syncState.get(1).getProtocolInformation());
+				DataObject o3 = o2.createCopy(false);
+				o3.setProtocolInformation(o2.getProtocolInformation());
+				expectedClientChanges.add(o3);
 				o2.commitChanges();
 				expectedSyncState.set(1, o2);
 				performSync(resolution, 0);
@@ -290,12 +349,12 @@ public class IncrementalContentSyncerTest extends TestCase {
 				syncState.clear();
 				dataOnServer.clear();
 				for (int i = 0; i < INITIAL_LARGE_DATAOBJECT_COUNT; i++) {
-					DataObject o = createDataObject("syncID" + i, "Sync" + i, 1, true);
+					DataObject o = createDataObject("syncID" + i, "Sync" + i, null, 1, true);
 					o.setUUID(UUID.randomUUID());
 					syncState.add(o);
-					dataOnServer.add(createDataObject("syncID" + i, "Server" + i, 2, true));
-					dataOnServer.add(createDataObject("syncID" + (i + INITIAL_LARGE_DATAOBJECT_COUNT), "New" + i, 2,
-							true));
+					dataOnServer.add(createDataObject("syncID" + i, "Server" + i, null, 2, true));
+					dataOnServer.add(createDataObject("syncID" + (i + INITIAL_LARGE_DATAOBJECT_COUNT), "New" + i, null,
+							2, true));
 				}
 				for (int i = 0; i < 2 * INITIAL_LARGE_DATAOBJECT_COUNT; i++) {
 					handler.resetServerCalls();
@@ -313,12 +372,12 @@ public class IncrementalContentSyncerTest extends TestCase {
 					int index = Integer.parseInt(clientChanges[0].getID().substring(6));
 					DataObject o = null;
 					if (i < INITIAL_LARGE_DATAOBJECT_COUNT) {
-						o = createDataObject("syncID" + index, "Sync" + index, i + 1, true);
+						o = createDataObject("syncID" + index, "Sync" + index, null, i + 1, true);
 						o.setTimestamp(i + 2L);
 						o.setFieldContent("Field2", "Server" + index);
 					} else {
-						o = createDataObject("syncID" + index, "New" + (index - INITIAL_LARGE_DATAOBJECT_COUNT), i + 2,
-								true);
+						o = createDataObject("syncID" + index, "New" + (index - INITIAL_LARGE_DATAOBJECT_COUNT), null,
+								i + 2, true);
 						o.setChangeState(ChangeState.CREATED);
 					}
 					expectedClientChanges.add(o.createCopy(false));
@@ -359,8 +418,10 @@ public class IncrementalContentSyncerTest extends TestCase {
 		o.commitChanges();
 		o.setTimestamp(2L);
 		o.setFieldContent("Field2", "ModifiedServer");
+		o.setProtocolInformation(syncState.get(1).getProtocolInformation());
 		expectedClientChanges.add(o);
 		o = o.createCopy(false);
+		o.setProtocolInformation(syncState.get(1).getProtocolInformation());
 		o.commitChanges();
 		expectedSyncState.set(1, o);
 		performSync(resolution, 0);
@@ -372,6 +433,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 		handler._writeUpdateToCall = true;
 		DataObject o = clientChanges.get(0).createCopy(false);
 		handler._objectToSend = o.createCopy(false);
+		o.setProtocolInformation(syncState.get(1).getProtocolInformation());
 		o.setTimestamp(2L);
 		o.commitChanges();
 		expectedSyncState.set(1, o);
@@ -408,6 +470,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 	private void clientDeletionServerChangeExpectChange(ConflictResolution resolution) throws USMException {
 		DataObject o = initClientDeletionServerChange();
 		DataObject o2 = o.createCopy(false);
+		o2.setProtocolInformation(o.getProtocolInformation());
 		o2.commitChanges();
 		expectedSyncState.set(1, o2);
 		o.rollbackChanges();
@@ -472,6 +535,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 		DataObject o2 = dataOnServer.get(1);
 		o2.setFieldContent("Field2", "ModifiedServer");
 		o = o2.createCopy(false);
+		o.setProtocolInformation(syncState.get(1).getProtocolInformation());
 		o2.commitChanges();
 		return o;
 	}
@@ -484,6 +548,7 @@ public class IncrementalContentSyncerTest extends TestCase {
 		DataObject o2 = dataOnServer.remove(1);
 		o2.setTimestamp(1L);
 		o2.commitChanges();
+		o2.setProtocolInformation(syncState.get(1).getProtocolInformation());
 		o2.setChangeState(ChangeState.DELETED);
 		expectedSyncState.remove(1);
 		return o2;
@@ -499,11 +564,13 @@ public class IncrementalContentSyncerTest extends TestCase {
 		}
 	}
 
-	private DataObject createDataObject(String id, String field2, long timestamp, boolean commit) {
+	private DataObject createDataObject(String id, String field2, ProtocolInformation protocolInfo, long timestamp,
+			boolean commit) {
 		DataObject o = contentType.newDataObject(session);
 		o.setID(id);
 		o.setFieldContent("Field2", field2);
 		o.setTimestamp(timestamp);
+		o.setProtocolInformation(protocolInfo);
 		if (commit)
 			o.commitChanges();
 		return o;
@@ -519,8 +586,8 @@ public class IncrementalContentSyncerTest extends TestCase {
 	private SyncResult performSync(ConflictResolution resolution, int expectedServerCalls, long timestamp, int limit,
 			boolean expectIncomplete) throws USMException {
 		syncerStorage.setCurrentServerData(DataObjectUtil.toArray(dataOnServer));
-		SyncResult result = contentSyncer.syncChangesWithServer(syncerStorage, resolution, log, DataObjectUtil
-				.toArray(syncState), DataObjectUtil.toArray(clientChanges), 1, limit);
+		SyncResult result = contentSyncer.syncChangesWithServer(syncerStorage, resolution, DataObjectUtil
+				.toArray(syncState), DataObjectUtil.toArray(clientChanges), 1, limit, null, null);
 		assertEquals(timestamp, result.getTimestamp());
 		assertEquals(expectIncomplete, result.isIncomplete());
 		handler.checkCalls(expectedServerCalls);
@@ -543,11 +610,12 @@ public class IncrementalContentSyncerTest extends TestCase {
 		expectedClientChanges.clear();
 		handler.resetServerCalls();
 		for (int i = 0; i < INITIAL_SMALL_DATAOBJECT_COUNT; i++) {
-			DataObject o = createDataObject("syncID" + i, "Sync" + i, 1, true);
+			ProtocolInformation info = new TestProtocolInfo("Existing" + i);
+			DataObject o = createDataObject("syncID" + i, "Sync" + i, info, 1, true);
 			o.setUUID(UUID.randomUUID());
 			syncState.add(o);
-			dataOnServer.add(createDataObject("syncID" + i, "Sync" + i, 2, true));
-			expectedSyncState.add(createDataObject("syncID" + i, "Sync" + i, 2, true));
+			dataOnServer.add(createDataObject("syncID" + i, "Sync" + i, null, 2, true));
+			expectedSyncState.add(createDataObject("syncID" + i, "Sync" + i, info, 2, true));
 		}
 		for (DataObject o : newServerObjects) {
 			dataOnServer.add(o.createCopy(false));
