@@ -53,8 +53,6 @@ package com.openexchange.usm.session.sync.test;
 
 import junit.framework.TestCase;
 
-import org.apache.commons.logging.Log;
-
 import com.openexchange.usm.api.exceptions.USMException;
 import com.openexchange.usm.api.session.*;
 import com.openexchange.usm.configuration.ConfigurationProperties;
@@ -73,12 +71,11 @@ public class SlowContentSyncerTest extends TestCase {
 	DataObject[] clientData = {};
 	DataObject[] dataOnServer = {};
 	SimContentSyncerStorage syncerStorage = new SimContentSyncerStorage();
-	SimJournal journal = new SimJournal();
-	Log log = journal.getLog(SlowContentSyncerTest.class);
 	SimSession session = new SimSession();
 	SimSyncContentType contentType = new SimSyncContentType();
 	SimSyncContentTypeTransferHandler handler = (SimSyncContentTypeTransferHandler) contentType.getTransferHandler();
-	SlowContentSyncer slowContentSyncer = new SlowContentSyncer(new SessionManagerImpl(),
+	SlowContentSyncer slowContentSyncer = new SlowContentSyncer(new SessionManagerImpl(new SimJournal()
+			.getLog(SlowContentSyncerTest.class)),
 			ConfigurationProperties.SYNC_CONCURRENT_MODIFICATION_MAX_RETRIES_DEFAULT);
 
 	public void testNoObjectOnClientNoObjectOnServerERROR() throws USMException {
@@ -110,7 +107,7 @@ public class SlowContentSyncerTest extends TestCase {
 		dataOnServer = new DataObject[0];
 		syncerStorage.setCurrentServerData(dataOnServer);
 		handler.resetServerCalls();
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(0);
 		assertEquals(0, result.getChanges().length);
 		assertEquals(0, syncerStorage.getServerDataToStore().length);
@@ -148,7 +145,7 @@ public class SlowContentSyncerTest extends TestCase {
 		handler.resetServerCalls();
 		handler._writeNewToCall = true;
 		handler._objectToSend = clientData[0];
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(1);
 		assertEquals(0, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -189,7 +186,7 @@ public class SlowContentSyncerTest extends TestCase {
 		syncerStorage.setCurrentServerData(dataOnServer);
 		handler.resetServerCalls();
 		contentType.setMatchRating(1);
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(0);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -239,7 +236,7 @@ public class SlowContentSyncerTest extends TestCase {
 		handler.resetServerCalls();
 		handler._writeNewToCall = true;
 		handler._objectToSend = clientData[0];
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(1);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(2, syncerStorage.getServerDataToStore().length);
@@ -288,7 +285,7 @@ public class SlowContentSyncerTest extends TestCase {
 		dataOnServer[0].commitChanges();
 		syncerStorage.setCurrentServerData(dataOnServer);
 		handler.resetServerCalls();
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(0);
 		assertEquals(0, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -333,7 +330,7 @@ public class SlowContentSyncerTest extends TestCase {
 		contentType.setMatchRating(1);
 		syncerStorage.setCurrentServerData(dataOnServer);
 		handler.resetServerCalls();
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(0);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -354,9 +351,9 @@ public class SlowContentSyncerTest extends TestCase {
 			throws USMException {
 		initConflictingObjectsWithSameID();
 		try {
-			slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+			slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 			fail("Conflicting changes exception not thrown!");
-		} catch (SynchronizationConflictException e) {
+		} catch (SynchronizationConflictException ignored) {
 
 		}
 	}
@@ -378,7 +375,7 @@ public class SlowContentSyncerTest extends TestCase {
 		handler._objectToSend = DataObjectUtil.copyAndModify(dataOnServer[0], clientData[0], false);
 		//		handler._objectToSend.setChangeState(ChangeState.MODIFIED);
 		//		handler._objectToSend.commitChanges();
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, resolution, 0, null, null, clientData);
 		handler.checkCalls(1);
 		assertEquals(0, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -398,7 +395,8 @@ public class SlowContentSyncerTest extends TestCase {
 			throws USMException {
 		initConflictingObjectsWithSameID();
 		DataObject expectedClientResult = DataObjectUtil.copyAndModify(clientData[0], dataOnServer[0], false);
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, 0, null, null,
+				clientData);
 		handler.checkCalls(0);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -432,9 +430,9 @@ public class SlowContentSyncerTest extends TestCase {
 			throws USMException {
 		initConflictingObjectWithDifferentIDs();
 		try {
-			slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, log, 0, clientData);
+			slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, 0, null, null, clientData);
 			fail("Conflicting changes exception not thrown!");
-		} catch (SynchronizationConflictException e) {
+		} catch (SynchronizationConflictException ignored) {
 
 		}
 	}
@@ -459,7 +457,8 @@ public class SlowContentSyncerTest extends TestCase {
 		DataObject expectedStorageResult = expectedClientChange.createCopy(false);
 		expectedStorageResult.commitChanges();
 		handler._objectToSend = serverChange;
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, 0, null, null,
+				clientData);
 		handler.checkCalls(1);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -479,7 +478,8 @@ public class SlowContentSyncerTest extends TestCase {
 			throws USMException {
 		initConflictingObjectWithDifferentIDs();
 		DataObject expectedClientResult = DataObjectUtil.copyAndModify(clientData[0], dataOnServer[0], false);
-		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, log, 0, clientData);
+		SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, conflictResolution, 0, null, null,
+				clientData);
 		handler.checkCalls(0);
 		assertEquals(1, result.getChanges().length);
 		assertEquals(1, syncerStorage.getServerDataToStore().length);
@@ -515,8 +515,8 @@ public class SlowContentSyncerTest extends TestCase {
 		}
 		syncerStorage.setCurrentServerData(dataOnServer);
 		for (int i = 0; i < MANY_DATAOBJECTS_COUNT; i++) {
-			SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, ConflictResolution.USE_SERVER, log, 1,
-					clientData);
+			SyncResult result = slowContentSyncer.syncWithServer(syncerStorage, ConflictResolution.USE_SERVER, 1, null,
+					null, clientData);
 			handler.checkCalls(0);
 			assertEquals(i < MANY_DATAOBJECTS_COUNT - 1, result.isIncomplete());
 			assertEquals(1, result.getChanges().length);
