@@ -25,7 +25,7 @@ public class DebugLogWriter implements DebugWriter {
 	}
 
 	public void log(String x) {
-		_journal.info(_descriptor + ' ' + x);
+		_journal.debug(_descriptor + ' ' + x);
 	}
 
 	public void close() throws IOException {
