@@ -15,7 +15,7 @@ public class DebugPrintWriter extends PrintWriter implements DebugWriter {
 
 	public DebugPrintWriter(String fileName) throws FileNotFoundException {
 		super(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fileName, true))), false);
-		log("EAS Servlet debug logging started");
+		log("SyncML Servlet debug logging started");
 		flush();
 	}
 
@@ -29,7 +29,7 @@ public class DebugPrintWriter extends PrintWriter implements DebugWriter {
 
 	@Override
 	public void close() {
-		log("EAS Servlet debug logging stopped");
+		log("SyncML Servlet debug logging stopped");
 		super.close();
 	}
 }
