@@ -1,7 +1,7 @@
 package com.openexchange.usm.syncml.servlet;
 
 public class HexDumper {
-	private final static String HEX_DIGITS = "0123456789ABCDEF";
+	private static final String HEX_DIGITS = "0123456789ABCDEF";
 
 	public static String toString(byte[] byteArray) {
 		StringBuilder sb = new StringBuilder(byteArray.length + 3);
@@ -13,7 +13,7 @@ public class HexDumper {
 					sb2.append(' ');
 				} else {
 					int b = ((int) byteArray[i + j]) & 0xFF;
-					sb.append(HEX_DIGITS.charAt(b / 16)).append(HEX_DIGITS.charAt(b % 16)).append(' ');
+					sb.append(HEX_DIGITS.charAt(b >> 4)).append(HEX_DIGITS.charAt(b % 16)).append(' ');
 					if ((b >= 32 && b <= 127) || (b >= 160))
 						sb2.append((char) b);
 					else
@@ -26,10 +26,10 @@ public class HexDumper {
 	}
 
 	public static String toShortString(byte[] byteArray) {
-		StringBuilder sb = new StringBuilder(byteArray.length*2);
+		StringBuilder sb = new StringBuilder(byteArray.length << 1);
 		for (int i = 0; i < byteArray.length; i++) {
 			int b = ((int) byteArray[i]) & 0xFF;
-			sb.append(HEX_DIGITS.charAt(b / 16)).append(HEX_DIGITS.charAt(b % 16));
+			sb.append(HEX_DIGITS.charAt(b >> 4)).append(HEX_DIGITS.charAt(b % 16));
 		}
 		return sb.toString();
 	}
