@@ -56,6 +56,7 @@ import java.text.SimpleDateFormat;
 import java.util.*;
 import java.util.Map;
 
+import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.commons.httpclient.HttpStatus;
@@ -81,7 +82,7 @@ import com.openexchange.usm.syncml.util.XmlSizeSupport;
 public class SyncMLRequest {
 	private static final String BASIC_AUTH_DELIMITER = ":";
 
-	private final static String DATE_FORMAT_STRING = "yyyyMMdd'T'HHmmssSSS'Z'";
+	private static final String DATE_FORMAT_STRING = "yyyyMMdd'T'HHmmssSSS'Z'";
 
 	// 3 fields for a sync state information: client_anchor, server_anchor and the internal timestamp used for the server anchor
 	// We store those 3 fields for 2 states, "last" and "next":
@@ -120,13 +121,16 @@ public class SyncMLRequest {
 	private int _maxMessageSize;
 
 	private Map<String, Folder> _folders = null;
+	
+	private HttpServletRequest _servletRequest;
 
-	public SyncMLRequest(SyncMLServlet servlet, XmlPullParser parser, XmlSerializer serializer, boolean wbxml) {
+	public SyncMLRequest(SyncMLServlet servlet, XmlPullParser parser, XmlSerializer serializer, boolean wbxml, HttpServletRequest servletRequest) {
 		_servlet = servlet;
 		_parser = parser;
 		_serializer = serializer;
 		_wbxml = wbxml;
 		_journal = servlet.getJournal();
+		_servletRequest = servletRequest;
 	}
 
 	public int handleRequest(HttpServletResponse resp) {
@@ -170,7 +174,7 @@ public class SyncMLRequest {
 					int maxMsgSize = Integer.parseInt(val);
 					if (maxMsgSize > 0)
 						return maxMsgSize;
-				} catch (NumberFormatException nfe) {
+				} catch (NumberFormatException ignored) {
 				}
 				if (_journal.isDebugEnabled())
 					_journal.debug(_usmSession + " Client device sent illegal MaxMsgSize of " + val);
@@ -272,7 +276,7 @@ public class SyncMLRequest {
 		// 3) check if authentication is valid, retrieve USM session
 		try {
 			_usmSession = _servlet.getSessionManager().getSession(user, password, SyncMLConstants.PROTOCOL_NAME,
-					device, _servlet.getSessionInitializer());
+					device, _servlet.getSessionInitializer(), _servletRequest.getRemoteAddr(), getXHeadersFromCurrentRequest());
 			return SyncMLStatusCode.AUTHENTICATED;
 		} catch (AuthenticationFailedException e) {
 			if (_journal.isDebugEnabled())
@@ -287,6 +291,17 @@ public class SyncMLRequest {
 			return SyncMLStatusCode.SERVER_FAILURE; // LATER: Provide other status codes based on exception ?
 		}
 	}
+	
+	public Map<String, String> getXHeadersFromCurrentRequest() {
+		Map<String, String> headers = new HashMap<String, String>();
+		Enumeration<?> headerNames = _servletRequest.getHeaderNames();
+		while (headerNames.hasMoreElements()) {
+			String headerName = (String) headerNames.nextElement();
+			if(headerName.toLowerCase().startsWith("x-"))
+				headers.put(headerName, _servletRequest.getHeader(headerName));
+		}
+		return headers;
+	}
 
 	private void addSyncHdrStatus(SyncMLStatusCode status) {
 		Chal chal = (status == SyncMLStatusCode.OK || status == SyncMLStatusCode.AUTHENTICATED) ? null : new Chal();
@@ -361,6 +376,10 @@ public class SyncMLRequest {
 		return _request.getSyncHdr().getMsgID();
 	}
 
+	public boolean isRequestFinal() {
+		return _request.getSyncBody().isFinal();
+	}
+
 	public Map<String, Folder> getFolders() throws USMException {
 		if (_folders == null)
 			_folders = FolderHierarchy.getInstance().getFolders(_usmSession);
@@ -407,7 +426,7 @@ public class SyncMLRequest {
 
 	public String getLastServerAnchor(String path) {
 		String result = _usmSession.getPersistentField(getPersistentKey(LAST_SERVER_ANCHOR_PREFIX, path));
-		return result.length() > 0 ? result : null;
+		return (result.length() > 0) ? result : null;
 	}
 
 	private void updateAnchorData(SyncInformation syncInfo) throws DatabaseAccessException, USMSQLException {
@@ -491,4 +510,13 @@ public class SyncMLRequest {
 			_usedSyncInformations.add(syncInfo);
 		return syncInfo;
 	}
+
+	@Override
+	public String toString() {
+		return "SyncMLRequest(" + getDescription(_request) + "->" + getDescription(_response) + ')';
+	}
+
+	private String getDescription(SyncML syncML) {
+		return (syncML == null) ? "ERROR" : syncML.getSyncBody().getDescription();
+	}
 }
